function [IRF,transition,dist_uncond,dist] = IRF_calculation(N,Nnext,z_prob,options,vars)
% Calculates IRF (expected value and certain percentiles)
% INPUTS:
% N : grid for employmentN
% Nnext : N'(N,z) with N in rows, and z in columns
% z_prob : (i,j) element is probability of z_j given z_i
% options : includes fields
%   IRF_no : number of periods that are calculated
%   zstart : vector with starting value(s) for z. If shorter than
%               IRF_no, remaining periods assume random transition
%   distinit : of vector initial distribution; first gridno elements 
%           correspond to first value of z, second gridno elements to
%           second  value of z, etc.
%   percentiles : vector with percentiles that should be calculated
% vars : cells with value of variables for IRF with N in rows, and z in columns
%
% OUTPUTS: 
% IRF : field with the expected value and a field with the percentiles for
%           each variable
% transition : transition matrix for aggregate state such that
%       dist_nextperiod=transition'*dist_thisperiod
% dist_uncond : unconditional distribution calculated from transition matrix
%               with N in rows, and z in columns
% dist : matrix, with aggregate distribution, one column per period

IRF_no=options.IRF_no;
zstart=options.zstart;
distinit=options.distinit;
percentiles=options.percentiles;

grid_Nno = length(N); % number of grid points for N
z_no = size(z_prob,1); % number of grid points for z

% build transition matrix in terms of N and z
transition = sparse(grid_Nno*z_no,grid_Nno*z_no);

% interpolate N next period
temp = interp1(N,1:grid_Nno,Nnext,'linear','extrap');
temp_ind = max(1,min(grid_Nno-1,floor(temp))); % index of element next period
temp_weight = 1-(temp-temp_ind); % weight on index

for i=1:z_no
    % for each z, calculate transition in terms of N
    temp_transition = sparse([1:grid_Nno,1:grid_Nno],[temp_ind(:,i);temp_ind(:,i)+1],[temp_weight(:,i);1-temp_weight(:,i)],grid_Nno,grid_Nno);
    for j=1:z_no
        % for each z', multiply by transition probability
        transition((i-1)*grid_Nno+(1:grid_Nno),(j-1)*grid_Nno+(1:grid_Nno)) = z_prob(i,j)*temp_transition;
    end
end

[tempdist,~] = eigs(transition',1,1.0001);
dist_uncond = reshape(tempdist/sum(tempdist),grid_Nno,z_no); % unconditional distribution



dist = zeros(grid_Nno*z_no,IRF_no); % store for distribution in each period of IRF
dist(:,1) = distinit; % initial distribution

if abs(sum(distinit((zstart(1)-1)*grid_Nno+(1:grid_Nno)))-1) > 1e-12
    % check whether initial distribution corresponding to initial z sums to 1
    disp('Warning: initial distribution does not sum to 1!')
end

% deterministic transition for as many periods as starting values for z are
% given
for i=2:length(zstart)
    dist((zstart(i)-1)*grid_Nno+(1:grid_Nno),i) = 1/z_prob(zstart(i-1),zstart(i))*transition((zstart(i-1)-1)*grid_Nno+(1:grid_Nno),(zstart(i)-1)*grid_Nno+(1:grid_Nno))'*dist((zstart(i-1)-1)*grid_Nno+(1:grid_Nno),i-1);
end

% use full transition matrix, if z' is not given
for i=(length(zstart)+1):IRF_no
    dist(:,i) = transition'*dist(:,i-1);
end

for k=1:length(vars) % for each variable calculate average and perecentiles
    IRF.mean{k} = vars{k}(:)'*dist; % average

    for i=1:IRF_no
        temp = sortrows([vars{k}(:),dist(:,i)]); % sort distribution increasing in value
        temp_val = temp(:,1); % value of variable
        temp_p = cumsum(temp(:,2)); % CDF at value
        for j =1:length(percentiles)
            tempind = min(grid_Nno-1,find(percentiles(j)>temp_p,1,'last'));
            if isempty(tempind)
                tempind = 1;
            end
            tempweight = max(0,min(1,1-(percentiles(j)-temp_p(tempind))/(temp_val(tempind+1)-temp_val(tempind))));
            IRF.percentiles{k}(j,i) = tempweight*temp_val(tempind)+(1-tempweight)*temp_val(tempind+1);
        end 
    end

end
