function [dev,sim_results,sim_final] = alternativepath(params,coeffs,options,init,altJ,altP,altPb)
% Solves household problem and simulates economy along an alternative path.
% While on this path, agents expect to either stay on it or switch back to
% the infinite-horizon solution with given probabilities. (In the last
% period staying on the path is not an option).
%
% INPUTS:
%  params: structure with parameters
%
%  coeffs: coefficients of aggregate laws in row vector, blocks of z_no 
%  elements each, relevant after switching back:
%        1st: constants for J
%        2nd: linear coefficients for J
%        3rd: constants for P
%        4th: linear coefficients for P
%        5th: quadratic coefficients for J
%        6th: quadratic coefficients for P
%
%  options: structure with various "options" specified
%       policy_name: name of .mat-file with initial guesses for policy
%        functions and grid for N
%       HHcrit: accuracy of household problem
%       timefactor: time allowed for convergence of household problem
%       hist_grid: grid for histogram of agents used to simulate the
%        economy
%       init_dist: first (second) column has mass of employed (unemployed)
%        in initial distribution of agents at each grid point of the
%        histogram
%       init_q: initial allocation of equity in the histogram of agents
%        (first column employmed, second column unemployed)
%       init_b: initial allocation of bonds in the histogram of agents
%        (first column employmed, second column unemployed)
%       sim_p: series of aggregate productivity states used for simulation
%       sim_no: number of periods simulated
%       sim_drop: number of initial periods dropped, when using simulated
%        data
%       sim_individual_no: number of agents when simulating them separately
%
%  init: structure with initial conditions for alternative path
%       altz: vector with productivity in each period
%       state_expected: matrix with transition probabilities in each period
%           (first z_no columns for switching to usual states, last column
%           for staying on the path - has to be 0 in last row)
%       dist: distribution of agents to begin simulation
%       sim_b: initial bond holdings of agents
%       sim_q: initial equity holdings of agents
%
%  altJ: vector with perceived price of equity along the alternative path
%  altP: vector with perceived price level along the alternative path
%
%  altPb: (optional) price of bond along alternative path, if not provided
%    usual interest rate rule is used
%
% OUTPUTS:
%  dev: matrix with deviation between perceived and market clearing prices
%    for J and P along alternativ path
%
%  sim_results: simulation results along alternative path
%   each column contains time series of one aggregate variable
%       1) productivity
%       2) employment
%       3) market-clearing real price of equity
%       4) perceived real price of equits
%       5) market-clearing price level
%       6) perceived price level
%       7) consumption
%       8) real wage
%       9) taxes
%       10) perceived price of zero-coupon one period nominal bond
%       11) perceived real dividend
%       12) labour market tightness
%       13) expected real return on holding bonds
%       14) expected real return on holding equity
%       15) expected inflation rate
%
%  sim_final: structure with simulated agents in the last period
%       dist: distribution of agents
%       b: bond holdings of agents
%       q: equity holdings of agents      
       
beta = params.beta; %   discount factor
chi = params.chi; %   scale of bonds in utility (chi*((b-b_min)/P/(1+i))^(-zeta))
zeta = params.zeta; %   curvature of bonds in utitily 
delta = params.delta; %   separation probability
gamma = params.gamma; %   curvature of marginal utility of consumption (c^(-gamma))
B = params.B; %   supply of nominal bonds (each repays 1 unit in t+1)
mu = params.mu; %   unemployment insurance (either constant or column vector with UI in each state)
phi0 = params.phi0; %   matching function: phi0*v^phi1*u^(1-phi1)
phi1 = params.phi1; %   matching function: phi0*v^phi1*u^(1-phi1)
psi = params.psi; %   cost of posting vacancy
q_min = params.q_min; %   minimum number of shares that has to be held
b_min = params.b_min; %   minimum number of bonds that has to be held
IR_coeff = params.IR_coeff; %   Interest rate rule: 1+i_t+1 = max(IR_coeff(3),IR_coeff(1)*(P/meanP)^IR_coeff(2))
w_coeff = params.w_coeff; %   real wage equation: w_coeff(1) * (z/meanz)^w_coeff(2) * (P/meanP)^(w_coeff(3)-1)
z = params.z; %   row vector with possible values of productivity
z_prob = params.z_prob; %   z_prob(i,j) is probability of moving from state i to state j
meanN = params.meanN; % average employment used in laws of motion for P and J such that changing the slope coefficients doesn't affect the mean
elas = params.elas; % elasticity used to impose market clearing

policy_name=options.policy_name; % savename for policy functions
HHcrit = options.HHcrit; % accuracy of household problem
timefactor = options.timefactor; % increase to allow for more time solving the household problem

load(policy_name) % load initial guess for policy functions


z_no = length(z); % number of aggregate states
grid_Nno = length(grid_N); % number of grid points on grid for N
x_no = length(x_grid); % numer of grid points for financial wealth


if length(coeffs) == 4*z_no % only linear law used
    coeffs(4*z_no+(1:(2*z_no))) = 0; % set quadratic coefficients 0 and estimate linear law in the end
    lawtype = 'linear';
elseif length(coeffs) == 6*z_no % quadratic law
    lawtype = 'quadratic';
elseif length(coeffs) == 10 % constant log(z) log(N/meanN) log(z)^2 log(N/meanN)^2 for log J and log P
    lawtype = 'logquadratic';
end

% Aggregate states summarized in S: {p,N}, where p is the number of the
% productivity state and N is the value of aggregate employment
% Compute some aggregate variables as functions of aggregate state:
if strcmp(lawtype,'logquadratic')
    aggJ = @(S) exp(coeffs(1) + coeffs(2)*log(z(S{1})) + coeffs(3)*log(S{2}/meanN) + coeffs(4)*log(z(S{1})).^2 + coeffs(5)*log(S{2}/meanN).^2); % real price of equity
    aggP = @(S) exp(coeffs(6) + coeffs(7)*log(z(S{1})) + coeffs(8)*log(S{2}/meanN) + coeffs(9)*log(z(S{1})).^2 + coeffs(10)*log(S{2}/meanN).^2); % price level
else % quadratic law
    aggJ = @(S) exp(coeffs(S{1})       +log(S{2}/meanN).*coeffs(z_no+S{1})  +log(S{2}/meanN).^2.*coeffs(4*z_no+S{1})); % real price of equity
    aggP = @(S) exp(coeffs(2*z_no+S{1})+log(S{2}/meanN).*coeffs(3*z_no+S{1})+log(S{2}/meanN).^2.*coeffs(5*z_no+S{1})); % price level
end
    
agglambdaf = @(S) psi*aggJ(S).^(-1); % filling probability of vacancies
aggtheta   = @(S) (phi0./agglambdaf(S)).^(1/(1-phi1)); % labor market tightness
agglambdaw = @(S) agglambdaf(S).*aggtheta(S); % transition probability for unemployed
aggNp = @(S) (1-delta)*S{2}+agglambdaw(S).*(1-S{2}); % employment next period


% Find long run values of employment in each state
Nlimit = zeros(1,z_no);
solver_options = optimset('Display','off','tolFun',1e-12,'tolX',1e-12);
for i=1:z_no
   Nlimit(i)=fsolve(@(x)aggNp({i,x})-x,0.9,solver_options); % long run value 
end
% find unconditional distribution over states in z
[tempv,~]=eigs(z_prob',1);
tempv=tempv/sum(tempv);

% average P is defined as the long-run value of P in each state weighted with the unconditional distribution
meanP = aggP({1:z_no,Nlimit})*tempv;

% average z is defined as the exponential of unconditional mean of log(z)
meanz = exp(log(z(1:z_no))*tempv);

aggwage = @(S) w_coeff(1) * (z(S{1})/meanz).^w_coeff(2) .* (aggP(S)/meanP).^(w_coeff(3)-1); % real wage
aggd = @(S) z(S{1})-aggwage(S); % real dividends



smalleps = 1e-10; % small number to ensure that consumption and bond holdings remain positive while iterating


%%%%%%%% Find policy functions and prices in alternative state
altz = init.altz; % productivity along alternative path
init_no = length(altz); % number of periods on alternative path

try
    if nargin < 7
        altPb = 1./max(IR_coeff(3),IR_coeff(1)*(altP/meanP).^IR_coeff(2)); % use standard interest rule for nominal price of bond = 1/(1+i)
    end
    
    altlambdaw = phi0^(1/(1-phi1))* (altJ/psi).^(phi1/(1-phi1)); % transition probability for unemployed
    
    altwage = w_coeff(1) * (init.altz/meanz).^w_coeff(2) .* (altP/meanP).^(w_coeff(3)-1); % real wage
    altd = init.altz-altwage; % real dividends

    altN = zeros(init_no+1,1); % employment along alternative path
    altN(1) = sum(sum(init.sim_q.*init.dist));
    for t=1:init_no
        altN(t+1) = (1-delta)*altN(t) + altlambdaw(t)*(1-altN(t));
    end
    
    % after tax real income of employed (aggregate income with balanced budget = w*N - (1-Pb)*B/P)
    altincome = (altwage.*altN(1:init_no)-(1-altPb)*B./altP)./(altN(1:init_no)+(1-altN(1:init_no))*mu); 
    
    altJnext = ones(init_no,z_no+1); % beginning of period value of equity next period in each possible state
    altPnext = ones(init_no,z_no+1); % price level next period in each possible state

    
    
    for t=init_no:(-1):1 % find policy functions going backwards
        znext_no = z_no+(t<init_no); % number of possible states next period

        tempN_index = min(find(altN(t+1)>=grid_N,1,'last'),grid_Nno-1); % index of aggregate employment next period
        tempN_weight = (altN(t+1)-grid_N(tempN_index+1))./(grid_N(tempN_index)-grid_N(tempN_index+1)); % weight of the distribution that moves to index


        tempJnext = zeros(znext_no,2,x_no,2); % beginning of period value of equity next period
        tempPnext = zeros(znext_no,2,x_no,2); % price level next period
        tempcnextpolicy = zeros(znext_no,x_no,2); % consumption next period

        % calculate consumption next period in all possible states
        for i=1:z_no % if switching back to standard state
            altJnext(t,i) = aggd({i,altN(t+1)})+aggJ({i,altN(t+1)})*(1-delta); % beginning of period value of equity next period
            altPnext(t,i) = aggP({i,altN(t+1)}); % price level next period
            tempJnext(i,:,:,:) = altJnext(t,i); % beginning of period value of equity next period
            tempPnext(i,:,:,:) = altPnext(t,i); % price level next period

            tempcnextpolicy(i,:,:) = squeeze(tempN_weight*aggc(i,tempN_index,:,:) + (1-tempN_weight)*aggc(i,tempN_index,:,:)); % consumption next period
        end
        if t<init_no % continuation value on alternative path
            i=z_no+1;
            altJnext(t,i) = altd(t+1)+altJ(t+1)*(1-delta);
            altPnext(t,i) = altP(t+1);
            tempJnext(i,:,:,:) = altJnext(t,i); % beginning of period value of equity next period
            tempPnext(i,:,:,:) = altPnext(t,i); % price level next period

            tempcnextpolicy(i,:,:) = altc{t+1};
        end    

        interpz_index = repmat(reshape(1:znext_no,[znext_no,1,1,1]),[1,2,x_no,2]); % next period's productivity
        interpeps_index = repmat(reshape([1,2],[1,2,1,1]),[znext_no,1,x_no,2]); % next period's employment status

        % transition probability including aggregate and idiosyncratic transition
        tempprobL = ones(znext_no,2,x_no,2);
        % idiosyncratic transition
        tempprobL(:,1,:,1) = tempprobL(:,1,:,1)*(1-delta); % E-E
        tempprobL(:,2,:,1) = tempprobL(:,2,:,1)*delta; % E-U
        tempprobL(:,1,:,2) = tempprobL(:,1,:,2)*altlambdaw(t); % U-E
        tempprobL(:,2,:,2) = tempprobL(:,2,:,2)*(1-altlambdaw(t)); % U-U
        % aggregate transition
        for i=1:znext_no % agg state next period
            tempprobL(i,:,:,:) = init.state_expected(t,i)*tempprobL(i,:,:,:);
        end  

        tempincome = altincome(t)*repmat([1,mu],x_no,1); % income of employed and unemployed
        tempx = repmat(x_grid',1,2); % financial wealth beginning of period

        HHdampening = 0.05;
        d2 = 1;    
        tic
        while d2 > HHcrit
            tempq = repmat(reshape(altq{t},[1,1,x_no,2]),[znext_no,2,1,1]); % equity holdings
            tempb = repmat(reshape(altb{t},[1,1,x_no,2]),[znext_no,2,1,1]); % bond holdings

            % calculate real financial wealth and consumption next period
            tempxnext = tempJnext.*tempq + tempb./tempPnext;

            % interpolation with respect to cash on hand next period
            temp = interp1(x_grid,1:x_no,tempxnext,'linear','extrap');
            interpx_index = max(1,min(floor(temp),x_no-1));
            interpx_weight = 1+interpx_index-temp;

            % consumption next period
            tempcnext = max(smalleps,interpx_weight.*tempcnextpolicy(sub2ind([znext_no,x_no,2],interpz_index,interpx_index,interpeps_index))+...
                (1-interpx_weight).*tempcnextpolicy(sub2ind([znext_no,x_no,2],interpz_index,interpx_index+1,interpeps_index)));


            % Marginal utility of consumption next period
            tempMUcnext = tempcnext.^(-gamma);

            % Calculate expected marginal utility on the right hand side of FOC for equity
            tempEMUq = beta*squeeze(sum(sum(tempprobL.*tempMUcnext.*tempJnext,1),2))/altJ(t);

            % Calculate expected marginal utility on the right hand side of FOC for bonds
            tempEMUb = beta*squeeze(sum(sum(tempprobL.*tempMUcnext./tempPnext,1),2))*altP(t)/altPb(t);
            if chi>0 % add marginual utility of holding bonds
                tempEMUb = tempEMUb + chi*((altb{t}-b_min)/(altP(t)/altPb(t))).^(-zeta);
            end

            % consumption from FOC that is binding or equal to maximum consumption
            tempc_new = min(tempincome+tempx-altJ(t)*q_min-altPb(t)/altP(t)*b_min,max(smalleps,max(tempEMUq,tempEMUb)).^(-1/gamma));

            % adjust demand for equity proportional to total amount invested and adjust it 
            % downwards (upwards), if right hand side of FOC for equity (tempEMUq) is smaller
            % (larger) than right hand side of FOC for money (tempEMUb)
            tempq_new = max(q_min,min((tempincome+tempx-tempc_new-altPb(t)/altP(t)*b_min-chi*smalleps*10^5)./altJ(t),q_min+(altq{t}-q_min).*(tempincome+tempx-tempc_new-altJ(t)*q_min-altPb(t)/altP(t)*b_min)./(tempincome+tempx-altc{t}-altJ(t)*q_min-altPb(t)/altP(t)*b_min)+100*log(tempEMUq./tempEMUb)*meanz));
            tempb_new = (tempincome+tempx-tempc_new-altJ(t)*tempq_new)*altP(t)/altPb(t); % demand for bonds from budget constraint

            % Calculate differences between new and old policy functions
            d1_c = abs(tempc_new-altc{t})./(1+altc{t});
            d1_c_max = max(d1_c(:));
            d1_b = abs(tempb_new-altb{t})./(1+altb{t});
            d1_b_max = max(d1_b(:));
            d1_q = abs(tempq_new-altq{t})./(1+altq{t});
            d1_q_max = max(d1_q(:));
            d2 = max([d1_c_max,d1_b_max,d1_q_max]);

            % update policy functions
            dampening = 100/(100+max(0,toc/timefactor-5)^2)*HHdampening; % dampen more after longer running time
            altc{t} = altc{t}+dampening*(tempc_new-altc{t});
            altb{t} = altb{t}+dampening*(tempb_new-altb{t});
            altq{t} = altq{t}+dampening*(tempq_new-altq{t});

            if toc>120*timefactor % stop iterating, if it hasn't converged by that time
                disp(['Household problem: time out, period: ',num2str(t),' error: ',num2str(d2)])
                break
            end

        end    


    end

    sim_no = init_no;
    hist_grid = options.hist_grid; % grid for histogram of agents used to simulate the economy
    hist_no = length(hist_grid);
    sim_dist = init.dist; % distribution
    sim_q = init.sim_q; % last period's equity holdings
    sim_b = init.sim_b; % last period's bond holdings

    % productivity and perceived prices according to provided values
    sim_z=altz;
    sim_N=altN;
    sim_Jp=altJ;
    sim_Pp=altP;
    sim_Pbp=altPb;
    sim_w=altwage;

    sim_J=zeros(sim_no,1);
    sim_P=zeros(sim_no,1);
    sim_Erb=zeros(sim_no,1);
    sim_Erq=zeros(sim_no,1);
    sim_Einf=zeros(sim_no,1);

    sim_tax=zeros(sim_no,1);
    sim_dp=zeros(sim_no,1);
    sim_Bp=zeros(sim_no,1);
    sim_Q=zeros(sim_no,1);
    sim_Qp=zeros(sim_no,1);
    sim_Cp=zeros(sim_no,1);
    sim_theta=zeros(sim_no,1);
    sim_income=zeros(sim_no,2);

    for t=1:sim_no % Simulate
        % Interpolate policy functions on finer grid
        for i=1:2
            sim_qp(:,i) = max(q_min,interp1(x_grid,altq{t}(:,i),hist_grid,'linear','extrap')); % individual demand for equity
            sim_bp(:,i) = max(b_min,interp1(x_grid,altb{t}(:,i),hist_grid,'linear','extrap')); % individual demand for nominal bonds
            sim_cp(:,i) = interp1(x_grid,altc{t}(:,i),hist_grid,'linear','extrap'); % individual demand for consumption
        end

        sim_tax(t) = 1-(sim_N(t)*sim_w(t)-(1-sim_Pbp(t))*B/sim_Pp(t))/((1-sim_N(t))*mu+sim_N(t))/sim_w(t); % tax rate
        sim_income(t,:) = (1-sim_tax(t))*sim_w(t)*[1,mu]; % after tax income
        sim_dp(t) = sim_z(t)-sim_w(t); % perceived real dividend

        sim_Erq(t) = init.state_expected(t,:)*altJnext(t,:)'/sim_Jp(t)-1; % expected return on equity
        sim_Einf(t) = init.state_expected(t,:)*(altPnext(t,:)'/sim_Pp(t))-1; % expected inflation rate
        sim_Erb(t) = init.state_expected(t,:)*(sim_Pp(t)./altPnext(t,:)')/sim_Pbp(t)-1; % expected real return on bonds

        for i=1:2 % current employment status
            % perceived financial wealth conditional on last period's and this period's employment
            sim_xp(i,:,:) = ((1-delta)*sim_Jp(t)+sim_dp(t))*sim_q+sim_b/sim_Pp(t);
        end

        % transition of histogram (including transition in labour market)
        sim_lambdaw = (sim_N(t)-(1-delta)*sum(sim_dist(:,1)))/sum(sim_dist(:,2)); % job-finding probability implied by current employment and employment last period
        sim_transition = [1-delta,delta;sim_lambdaw,1-sim_lambdaw]; % transition probabilities 

        % interpolation of financial wealth
        tempindex = interp1(hist_grid,1:hist_no,sim_xp,'linear','extrap');
        hist_index = max(1,min(floor(tempindex),hist_no-1));
        hist_weight = max(0,min(1,(sim_xp-hist_grid(hist_index+1))./(hist_grid(hist_index)-hist_grid(hist_index+1)))); % weight of the distribution that moves to index

        sim_dist_new = zeros(hist_no,2); % distribution in the beginning of the period
        % add weight for each combination of this period's employment, last
        % period's employment, and last period's financial wealth
        for i=1:2 % employment this period
            for j=1:2 % employment last period
                for k=1:hist_no % grid point of financial wealth last period
                    sim_dist_new(hist_index(i,k,j),i) =   sim_dist_new(hist_index(i,k,j),i) + hist_weight(i,k,j)*sim_dist(k,j)*sim_transition(j,i);
                    sim_dist_new(hist_index(i,k,j)+1,i) = sim_dist_new(hist_index(i,k,j)+1,i) + (1-hist_weight(i,k,j))*sim_dist(k,j)*sim_transition(j,i);
                end
            end
        end


        sim_Bp(t) = sum(sum(sim_bp.*sim_dist_new)); % aggregate demand for bonds
        sim_Qp(t) = sum(sum(sim_qp.*sim_dist_new)); % aggregate demand for equity
        sim_Cp(t) = sum(sum(sim_cp.*sim_dist_new)); % aggregate demand for consumption

        % market clearing in the bond market
        if sim_Bp(t)>B/1000
            sim_P(t) = ((B-b_min)/(sim_Bp(t)-b_min))^(1/elas)*sim_Pp(t);    % market clearing price
            sim_b = b_min+max(0,(sim_bp-b_min)*(sim_P(t)/sim_Pp(t))^elas);       % allocation of bonds
        else
            % if there is no demand for bonds, distribute supply evenly (only happens, if aggregate laws are too far off)
            sim_P(t)=10*sim_Pp(t);
            sim_b = B*ones(hist_no,2);
        end

        try
            % market clearing price in the equity market: aggregate supply
            % given from matching function must be equal to aggregate demand
            if elas==1 && phi1 == 0.5 % analytical solution for J possible
                sim_J(t) = (-(1-delta)*sim_N(t)+sqrt(((1-delta)*sim_N(t))^2+4*phi0^2/psi*(1-sim_N(t))*sim_Qp(t)*sim_Jp(t)))/(2*phi0^2/psi*(1-sim_N(t)));
            else
                solver_options = optimset('Display','off','tolFun',1e-12,'tolX',1e-12);
                sim_J(t) = fsolve(@(J)(1-delta)*sim_N(t)+phi0*(psi/phi0/J).^(phi1/(phi1-1))*(1-sim_N(t))-sim_Qp(t)*(sim_Jp(t)/J)^elas,sim_Jp(t),solver_options);            
            end
        catch
            sim_J(t) = 0;
        end

        % always distribute equity according to perceived price of equity
        sim_Q(t) = sim_N(t+1);
        if sim_J(t)>0.01 && sim_Qp(t)>0.01
            sim_q = q_min+max(0,(sim_qp-q_min)*(sim_Q(t)-q_min)/(sim_Qp(t)-q_min));
        else
            % if there is no demand for equity, distribute supply evenly (only happens, if aggregate laws are too far off)
            sim_q = max(q_min,sim_Q(t)*ones(hist_no,2));
        end

        sim_theta(t) = (sim_J(t)/psi*phi0)^(1/(1-phi1)); % labor market tightness


        sim_dist = sim_dist_new; % update distribution
    end % end of simulation

    sim_C = sim_z.*sim_N(1:sim_no)-psi*sim_theta.*(1-sim_N(1:sim_no)); % aggregate consumption

    sim_results=[sim_z,sim_N(1:sim_no),sim_J,sim_Jp,sim_P,sim_Pp,sim_C,sim_w,sim_tax,sim_Pbp,sim_dp,sim_theta,sim_Erb,sim_Erq,sim_Einf];

    sim_final.dist = sim_dist;
    sim_final.q = sim_q;
    sim_final.b = sim_b;

    dev = [sim_J-sim_Jp,sim_P-sim_Pp]; % deviation between perceived and market clearing J and P

    % save policy functions
    save(policy_name,'grid_N','x_grid','aggb','aggq','aggc','altb','altq','altc')

    % remove comment to draw perceived and market clearing prices
    %{
    subplot(1,2,1)
    plot(1:init_no,sim_results(:,[3,4]),'.','MarkerSize',10)
    subplot(1,2,2)
    plot(1:init_no,sim_results(:,[5,6]),'.','MarkerSize',10)
    drawnow;
    %}

catch
    disp('Warning: error while calculating alternative path!')
    dev=NaN*ones(init_no,2);
    sim_results=NaN;
    sim_final=NaN;
end