function figures = figures_crosssection(ind_simulation,sim_results,params,options)
% Analyze cross section and provide values for figures 2-9
%
% INPUTS:
%  ind_simulation: structure that is provided in sim_details of the
%    het_aggrisk function, containing the simulated values of a cross-section
%    of individual agents
%  sim_results: corresponding time series of aggregate variables
%  params: structure with parameters
%  options: structure with options needed to solve heterogeneous agents
%
% OUTPUT:
%  figures: structure with values for figures 2-9
%   fig2: average consumption drop relative to last period of employment before and during unemployment spells
%   fig3: histogram of duration of unemployment spells
%   fig4: CDF of asset holdings relative to expected income loss at beginning of unemployment spell.
%   fig5: scatter plot of real cash-on-hand and relative consumption drop in first period of unemployment spell 
%   fig6: scatter plot of financial assets and share of bond holdings therein
%   fig7: scatter plot of real cash-on-hand and value of bonds
%   fig8: average real equity holdings before and during unemployment spells
%   fig9: average real bond holdings before and during unemployment spells


% 
series_names = {'consumption level','real bond holdings','value of equity holdings','relative consumption'};

plotpoints = options.plotpoints; % subsample of points drawn for each curve when plotting cross-sectional observations
sim_no = options.sim_no; % number of periods simulated
sim_drop = options.sim_drop; % number of initial periods dropped
sim_individual_no = options.sim_individual_no; % number of agents simulated


% get time series of aggregate variables (perceived) from sim_results and replicate them for all individuals
sim_z = repmat(sim_results(:,1),1,sim_individual_no); % productivity
sim_income = repmat(sim_results(:,8).*(1-sim_results(:,9)),1,sim_individual_no); % after tax real wage
sim_Pb_real = repmat(sim_results(:,10)./sim_results(:,6),1,sim_individual_no); % real price of bond
sim_J = repmat(sim_results(:,4),1,sim_individual_no); % real price of equity
sim_P = repmat(sim_results(:,6),1,sim_individual_no); % price level
sim_coh = ind_simulation.fwp+ sim_income.*((ind_simulation.emp==1) + params.mu*(ind_simulation.emp==2)) ; % real COH
sim_b = ind_simulation.bp.*sim_Pb_real; % real value of bonds purchased
sim_q = ind_simulation.qp.*sim_J; % real value of equity purchased
sim_assets = sim_b+sim_q; % total amount invested
sim_bshare = sim_b./sim_assets; % share of bonds in portfolio chosen
sim_JFR = params.phi0*(sim_J/params.psi*params.phi0).^(params.phi1/(1-params.phi1)); % job-finding rate


% calculate duration of employment / unemployment spells 
sim_duration = zeros([sim_no,sim_individual_no,4]);
% first (second) column calculate how many periods the agent has been
% employed (unemployed) in the current spell including this period
% third (fourth) column calculate how many periods the agent will still be 
% employed (unemployed) in the current spell including this period
% (e.g. 1 means that there will be a change of employment status after this 
% period)

for i=2:sim_no % assume that all spells start in second period
    % if agent is employed add 1 to length of employment spell - otherwise it remains 0
    sim_duration(i,:,1) = (sim_duration(i-1,:,1)+1).*(ind_simulation.emp(i,:) == 1);
    % if agent is unemployed add 1 to length of unemployment spell - otherwise it remains 0
    sim_duration(i,:,2) = (sim_duration(i-1,:,2)+1).*(ind_simulation.emp(i,:) == 2);
end

for i=(sim_no-1):(-1):1 % assume that all spells end in last period
    % if agent is employed add 1 to length of employment spell
    sim_duration(i,:,3) = (sim_duration(i+1,:,3)+1).*(ind_simulation.emp(i,:) == 1);
    % if agent is unemployed add 1 to length of unemployment spell
    sim_duration(i,:,4) = (sim_duration(i+1,:,4)+1).*(ind_simulation.emp(i,:) == 2);
    
end


for ind_series = 1:length(series_names)
    % pick series, drop first and last sim_drop periods to avoid having
    % spells, whose beginning or end is outside of the whole sample
    switch ind_series
        case 1
            tempseries = ind_simulation.cp((sim_drop+1):(sim_no-sim_drop),:); % consumption
        case 2
            tempseries = ind_simulation.bp((sim_drop+1):(sim_no-sim_drop),:).*sim_Pb_real((sim_drop+1):(end-sim_drop),:); % real value of bonds
        case 3
            tempseries = ind_simulation.qp((sim_drop+1):(sim_no-sim_drop),:).*sim_J((sim_drop+1):(end-sim_drop),:); % real value of equity
        case 4
            % calculate consumption in percent change to last period of previous
            % employment during unemployment spell
            % and percent change to last (future) period of employment
            % during employment spell
            
            
            % find relative consumption during unemployment spell
            temprelconsumption = zeros(sim_no,sim_individual_no);
            tempcemp = ones(1,sim_individual_no); % temporary variable that stores consumption at end of last employment spell
            for i = 2:sim_no
                tempnewspell = (ind_simulation.emp(i-1,:) == 1).*(ind_simulation.emp(i,:)==2); % identify new unemployment spells
                tempcemp = tempcemp.*(1-tempnewspell) + ind_simulation.cp(i-1,:).*tempnewspell; % update consumption at end of last employment spell, if a new spell begins
                temprelconsumption(i,:) = temprelconsumption(i,:).*(ind_simulation.emp(i,:)==1)+(ind_simulation.cp(i,:)./tempcemp-1).*(ind_simulation.emp(i,:)==2); % update relative consumption drop
            end
            
            % find relative consumption during employment spell            
            tempcemp = ones(1,sim_individual_no); % consumption at end of current employment spell
            for i =(sim_no-1):(-1):1
                tempnewspell = (ind_simulation.emp(i+1,:) == 2).*(ind_simulation.emp(i,:)==1); % identify end of employment spell
                tempcemp = tempcemp.*(1-tempnewspell) + ind_simulation.cp(i,:).*tempnewspell; % update consumption at end of current employment spell, if a new spell begins
                temprelconsumption(i,:) = temprelconsumption(i,:).*(ind_simulation.emp(i,:)==2)+(ind_simulation.cp(i,:)./tempcemp-1).*(ind_simulation.emp(i,:)==1); % update relative consumption drop
            end

            tempseries = temprelconsumption((sim_drop+1):(end-sim_drop),:);    
    end

    
    tempduration = sim_duration((sim_drop+1):(sim_no-sim_drop),:,:); % drop periods of employment / unemployment duration variables


    % calculate mean and quartiles of employed and unemployed at a certain
    % period into the respective spell
    periods = 10; % number of periods before and after unemployment shock, for which statistics are calculated

    stats_U = zeros(periods,5); % unemployed after job loss
    stats_E = zeros(periods,5); % employed before job loss
    stats_UB = zeros(periods,5); % unemployed, when first period of unemployment was boom
    stats_EB = zeros(periods,5); % employed, when first period of employment was boom
    stats_UR = zeros(periods,5); % unemployed, when first period of unemployment was recession
    stats_ER = zeros(periods,5); % employed, when first period of employment was recession


    for i=1:periods
        tempval = tempseries(squeeze(tempduration(:,:,2)==i)); % unemployed agents in i-th period of unemployment spell
        stats_U(i,:) = [mean(tempval),quantile(tempval,[0.25,0.5,0.75]),length(tempval(:))]; % mean, 25%, 50%, 75% quantiles, number of observations

        tempval = tempseries(squeeze(tempduration(:,:,3)==i)); % employed agents in i-th period before unemployment spell
        stats_E(i,:) = [mean(tempval),quantile(tempval,[0.25,0.5,0.75]),length(tempval(:))]; % mean, 25%, 50%, 75% quantiles, number of observations

 
        % subsample when first period of unemployment spell in boom
        tempval = tempseries(squeeze(tempduration(:,:,2)==i) & (sim_z(((sim_drop+1):(sim_no-sim_drop))-i+1,:)>1)); % during unemployment spell
        stats_UB(i,:) = [mean(tempval),quantile(tempval,[0.25,0.5,0.75]),length(tempval)]; % mean, 25%, 50%, 75% quantiles, number of observations

        tempval = tempseries(squeeze(tempduration(:,:,3)==i) & (sim_z(((sim_drop+1):(sim_no-sim_drop))+i,:)>1)); % before unemployment spell
        stats_EB(i,:) = [mean(tempval),quantile(tempval,[0.25,0.5,0.75]),length(tempval)]; % mean, 25%, 50%, 75% quantiles, number of observations

        % subsample when first period of unemployment spell in recession
        tempval = tempseries(squeeze(tempduration(:,:,2)==i) & (sim_z(((sim_drop+1):(sim_no-sim_drop))-i+1,:)<1)); % during unemployment spell
        stats_UR(i,:) = [mean(tempval),quantile(tempval,[0.25,0.5,0.75]),length(tempval)]; % mean, 25%, 50%, 75% quantiles, number of observations

        tempval = tempseries(squeeze(tempduration(:,:,3)==i) & (sim_z(((sim_drop+1):(sim_no-sim_drop))+i,:)<1)); % before unemployment spell
        stats_ER(i,:) = [mean(tempval),quantile(tempval,[0.25,0.5,0.75]),length(tempval)]; % mean, 25%, 50%, 75% quantiles, number of observations
    end

    j=1; % store mean of respective statistic
    
    % x-axis for figures: period of unemployment spell (0=last period of
    % employment)
    fig_x = repmat(((-periods+1):1:periods)',1,3);
    fig_y(:,1) = [stats_E(end:(-1):1,j);stats_U(:,j)]; % unconditional mean
    fig_y(:,2) = [stats_EB(end:(-1):1,j);stats_UB(:,j)]; % mean of spells starting in boom
    fig_y(:,3) = [stats_ER(end:(-1):1,j);stats_UR(:,j)]; % mena of spells starting in recessions
        
    if ind_series == 2 % save data for Figure 9 (real bond holdings)
        fig9.x = fig_x;
        fig9.y = fig_y;
        fig9.description = 'Average real bond holdings in given period before and during unemployment spells. x contains values for x-axis (period 0 = last period of employment. y contains unconditional average in first column; averages of agents in spells that started in a boom (recession) in second (third) column;';
    end

    if ind_series == 3 % save data for Figure 8 (real equity holdings)
        fig8.x = fig_x;
        fig8.y = fig_y;
        fig8.description = 'Average real equity holdings in given period before and during unemployment spells. x contains values for x-axis (period 0 = last period of employment. y contains unconditional average in first column; averages of agents in spells that started in a boom (recession) in second (third) column;';
    end

    if ind_series == 4 % relative consumption drop
        % save data for Figure 2 (relative consumption drop)
        fig2.x = fig_x;
        fig2.y = fig_y;
        fig2.description = 'Average consumption drop relative to last period of employment in given period before and during unemployment spells. x contains values for x-axis (period 0 = last period of employment. y contains unconditional average in first column; averages of agents in spells that started in a boom (recession) in second (third) column;';

        % Calculate drop in first period of unemployment
        tempdrop = tempseries(tempduration(:,:,2)==1); % drop of variables in first period of unemployment spell
        dummy_boom = sim_z((sim_drop+1):(sim_no-sim_drop),:)>1; % dummy=1 if in boom
        tempz1 = dummy_boom(tempduration(:,:,2)==1); % boom in first period of unemployment spell

        tempcoh = sim_coh((sim_drop+1):(sim_no-sim_drop),:);  % real COH
        tempx = tempcoh(tempduration(:,:,2)==1); % real COH in first period

        % scatter plot of cash-on-hand and change of variables in consumption
        % draw subsample
        [x1,y1] = subsample(tempx(tempz1==1),tempdrop(tempz1==1),plotpoints); % boom
        [x2,y2] = subsample(tempx(tempz1==0),tempdrop(tempz1==0),plotpoints); % recession

        % save data for Figure 5
        fig5.x1 = x1;
        fig5.x2 = x2;
        fig5.y1 = y1;
        fig5.y2 = y2;
        fig5.description = 'Relative consumption change in first period of unemployment spell given real cash-on-hand. x1 (x2) are observations of COH in boom (recession), y1 (y2) are observations of consumption change in boom (recession)';
        
    end

end


% calculate data for Figure 3
% histogram of duration of unemployment spells
templength = squeeze(tempduration(:,:,4)); % number of remaining quarters of unemployment spell
% take this number in first period of each unemployment spell = duration
tempcumdurationB = templength((tempduration(:,:,2)==1) & dummy_boom==1); % boom
tempcumdurationR = templength((tempduration(:,:,2)==1) & dummy_boom==0); % recession

fig3.x = 1:21; % length of completed unemployment spell, period 21 contains all spells that last more than 20 periods
fig3.y = [hist(tempcumdurationB,1:21)/length(tempcumdurationB);hist(tempcumdurationR,1:21)/length(tempcumdurationR)]'; % histogram of unemployment spells, normalized by total number of observations
fig3.description = 'Histogram of duration of completed unemployment spells. x contains length, y has spells starting in booms (recessions) in first (second) column';


% calculate data for Figure 6
% share of bonds in portfolio

tempbshare = sim_bshare((sim_drop+1):(sim_no-sim_drop),:); % share of bonds in portfolio
tempassets = sim_assets((sim_drop+1):(sim_no-sim_drop),:); % total amount invested
tempz = sim_z((sim_drop+1):(sim_no-sim_drop),:); % aggregate productivity
tempemp = ind_simulation.emp((sim_drop+1):(sim_no-sim_drop),:)==1; % employment dummy=1 if employed

[x1,y1] = subsample(tempassets(tempemp==1 & tempz>1),tempbshare(tempemp==1 & tempz>1),plotpoints); % employed in boom
[x2,y2] = subsample(tempassets(tempemp==0 & tempz>1),tempbshare(tempemp==0 & tempz>1),plotpoints); % unemployed in boom
[x3,y3] = subsample(tempassets(tempemp==1 & tempz<1),tempbshare(tempemp==1 & tempz<1),plotpoints); % employed in recession
[x4,y4] = subsample(tempassets(tempemp==0 & tempz<1),tempbshare(tempemp==0 & tempz<1),plotpoints); % unemployed in recession

% save data for Figure 6
fig6.x1 = x1;
fig6.x2 = x2;
fig6.x3 = x3;
fig6.x4 = x4;
fig6.y1 = y1;
fig6.y2 = y2;
fig6.y3 = y3;
fig6.y4 = y4;
fig6.description = 'scatter plot of financial assets (x) and share of bond holdings (y): 1. employed agents in boom, 2. unemployed agents in boom, 3. employed agents in recession, 4. unemployed agents in recession';


% calculate data for Figure 7
% real value of bonds

tempb = sim_b((sim_drop+1):(sim_no-sim_drop),:); % real value of bonds purchased
tempcoh = sim_coh((sim_drop+1):(sim_no-sim_drop),:);  % real COH
tempz = sim_z((sim_drop+1):(sim_no-sim_drop),:); % aggregate productivity
tempemp = ind_simulation.emp((sim_drop+1):(sim_no-sim_drop),:)==1; % employment dummy=1 if employed

[x1,y1] = subsample(tempcoh(tempemp==1 & tempz>1),tempb(tempemp==1 & tempz>1),plotpoints); % employed in boom
[x2,y2] = subsample(tempcoh(tempemp==0 & tempz>1),tempb(tempemp==0 & tempz>1),plotpoints); % unemployed in boom
[x3,y3] = subsample(tempcoh(tempemp==1 & tempz<1),tempb(tempemp==1 & tempz<1),plotpoints); % employed in recession
[x4,y4] = subsample(tempcoh(tempemp==0 & tempz<1),tempb(tempemp==0 & tempz<1),plotpoints); % unemployed in recession

% save data for Figure 7
fig7.x1 = x1;
fig7.x2 = x2;
fig7.x3 = x3;
fig7.x4 = x4;
fig7.y1 = y1;
fig7.y2 = y2;
fig7.y3 = y3;
fig7.y4 = y4;
fig7.description = 'scatter plot of real cash on hand (x) and value of bond holdings (y): 1. employed agents in boom, 2. unemployed agents in boom, 3. employed agents in recession, 4. unemployed agents in recession';


% calculate data for Figure 4
% distribution of assets relative to expected net wage loss
tempwage =  repmat(sim_results((sim_drop+1):(sim_no-sim_drop),8),1,sim_individual_no); % gross wage
tempz = sim_z((sim_drop+1):(sim_no-sim_drop),:); % aggregate productivity

% real value of assets beginning of period (excl. interest and dividend)
tempassetsbegin = ind_simulation.fwp((sim_drop+1):(sim_no-sim_drop),:)-ind_simulation.bp((sim_drop):(sim_no-sim_drop-1),:).*(1./repmat(sim_results((sim_drop):(sim_no-sim_drop-1),10),1,sim_individual_no)-1)./sim_P((sim_drop+1):(sim_no-sim_drop),:)-ind_simulation.qp((sim_drop):(sim_no-sim_drop-1),:).*(tempz-tempwage); 

tempJFR = sim_JFR((sim_drop+1):(sim_no-sim_drop),:); % job-finding rate
tempwageloss = sim_income((sim_drop+1):(sim_no-sim_drop),:)*(1-params.mu); % net wage loss

tempexpwageloss = mean(tempwageloss(:))*mean(1./tempJFR(:)); % expected wage loss*expected unemployment duration
temprelassets = tempassetsbegin./tempexpwageloss;
tempCDFnew = sort(temprelassets(tempduration(:,:,2)==1)); % CDF of newly unemployed

% save data for Figure 4
fig4.newx = tempCDFnew;
fig4.newy = (1:length(tempCDFnew))/length(tempCDFnew);
fig4.description = 'CDF of asset holdings relative to expected income loss for newly unemployed. newx:beginning of period real value of assets excl. dividends and interest payment, relative to (time-invariant) income loss = average wage loss * expected unemployment duration';



figures.fig2 = fig2;
figures.fig3 = fig3;
figures.fig4 = fig4;
figures.fig5 = fig5;
figures.fig6 = fig6;
figures.fig7 = fig7;
figures.fig8 = fig8;
figures.fig9 = fig9;



function [x,y] = subsample(x,y,n)
% Take random subsample of both vectors
tempindices = randperm(length(x));
x = x(tempindices(1:n));
y = y(tempindices(1:n));
