function [dev,sim_results,sim_details] = het_aggrisk(params,coeffs,options)
% Solves household problem with aggregate uncertainty and simulates economy
%
% INPUTS:
%  params: structure with parameters
%
%  coeffs: coefficients of aggregate laws in row vector, blocks of z_no 
%  elements each:
%        1st: constants for J
%        2nd: linear coefficients for J
%        3rd: constants for P
%        4th: linear coefficients for P
%        5th: quadratic coefficients for J
%        6th: quadratic coefficients for P
%
%  options: structure with various "options" specified
%       policy_name: name of .mat-file with initial guesses for policy
%        functions and grid for N
%       HHcrit: accuracy of household problem
%       timefactor: time allowed for convergence of household problem
%       hist_grid: grid for histogram of agents used to simulate the
%        economy
%       init_dist: first (second) column has mass of employed (unemployed)
%        in initial distribution of agents at each grid point of the
%        histogram
%       init_q: initial allocation of equity in the histogram of agents
%        (first column employmed, second column unemployed)
%       init_b: initial allocation of bonds in the histogram of agents
%        (first column employmed, second column unemployed)
%       sim_p: series of aggregate productivity states used for simulation
%       sim_no: number of periods simulated
%       sim_drop: number of initial periods dropped, when using simulated
%        data
%       sim_individual_no: number of agents when simulating them separately
%
% OUTPUTS:
%  dev: deviation between aggregate laws of motion given by coeffs and
%   estimated new laws using simulated data
%  sim_results: each column contains time series of one aggregate variable
%       1) productivity
%       2) employment
%       3) market-clearing real price of equity
%       4) perceived real price of equits
%       5) market-clearing price level
%       6) perceived price level
%       7) consumption
%       8) real wage
%       9) taxes
%       10) perceived price of zero-coupon one period nominal bond
%       11) perceived real dividend
%       12) labour market tightness
%       13) expected real return on holding bonds
%       14) expected real return on holding equity
%       15) expected inflation rate
%  sim_details: if requested, additional statistics are computed and saved
%  in this structure
%       1) cross-sectional averages for each period:
%           MRS: marginal rate of substitution E(c'/c)^-gamma
%           utility: life-time utility
%           X: (E[c'^-gamma*P/P'])^(1-zeta) needed for calculation of
%            shadow interest rate
%           bpower: bargaining power of employed
%           SIR: shadow nominal interest rate
%       2) ind_simulation: structure with time series of simulating individuals instead of
%       histogram, taking aggregate prices as given (one column for each
%       individual)
%             emp: employment status 1 if employed, 2 if unemployed
%             unempduration: duration of current unemployment spell or 0
%             fw: beginning of period real financial wealth
%             c: realized consumption
%             b: realized bond holdings
%             q: realized equity holdings
%             fwp: perceived beginning of period real financial wealth
%             income: after tax income (wage / unemployment insurance)
%             cp: perceived consumption
%             bp: perceived bond holdings
%             qp: perceived equity holdings
%             bpower: bargaining power
%             deposits2income: bond holdings to annualized labour income
%             assets2income: financial wealth to annualized labour income 
%             lastemp: employment status in last period
%             lastq: equity holdings of last period
%             lastb: bond holdings of last period
%       3) statistics averaged conditional on productivity states and unconditionally
%             c_drop4: average consumption drop during first 4 quarters of unemployment relative to last 8 quarters of employment
%             meanN: employment
%             assets2income: median financial wealth to annualized labour income 
%             unempduration: duration of unemployment spells
%       4) distribution and asset holdings of agents after many periods in each productivity state
%             store_dist: distribution
%             store_sim_q: equity holdings
%             store_sim_b: bond holdings
%       5) N_y_volatility: standard deviation of filtered log employment to std of log output
%       6) t_longrun: number of example periods after a long series in each productivity state
%       7) IRF: Impulse-response functions starting from good state drop to bad
%       state, each column contains time series of one aggregate variable
%           1) productivity
%           2) employment (percentage point-deviation)
%           3) output
%           4) real wage
%           5) real price of equity
%           6) price level
%           7) nominal interest rate (percentage point-deviation)
%           8) tax rate (percentage point-deviation)
%           9) tax volume
%           10) expected real interested rate (percentage point-deviation)
%           11) real unit labor costs
%           12) shadow nominal interest rate (percentage point-deviation)
%           13) shadow real interest rate (percentage point-deviation)
%           14) consumption
       
beta = params.beta; %   discount factor
chi = params.chi; %   scale of bonds in utility (chi*((b-b_min)/P/(1+i))^(-zeta))
zeta = params.zeta; %   curvature of bonds in utitily 
delta = params.delta; %   separation probability
gamma = params.gamma; %   curvature of marginal utility of consumption (c^(-gamma))
B = params.B; %   supply of nominal bonds (each repays 1 unit in t+1)
mu = params.mu; %   unemployment insurance (either constant or column vector with UI in each state)
phi0 = params.phi0; %   matching function: phi0*v^phi1*u^(1-phi1)
phi1 = params.phi1; %   matching function: phi0*v^phi1*u^(1-phi1)
psi = params.psi; %   cost of posting vacancy
q_min = params.q_min; %   minimum number of shares that has to be held
b_min = params.b_min; %   minimum number of bonds that has to be held
IR_coeff = params.IR_coeff; %   Interest rate rule: 1+i_t+1 = max(IR_coeff(3),IR_coeff(1)*(P/meanP)^IR_coeff(2))
w_coeff = params.w_coeff; %   real wage equation: w_coeff(1) * (z/meanz)^w_coeff(2) * (P/meanP)^(w_coeff(3)-1)
z = params.z; %   row vector with possible values of productivity
z_prob = params.z_prob; %   z_prob(i,j) is probability of moving from state i to state j
meanN = params.meanN; % average employment used in laws of motion for P and J such that changing the slope coefficients doesn't affect the mean
elas = params.elas; % elasticity used to impose market clearing

policy_name=options.policy_name; % savename for policy functions
HHcrit = options.HHcrit; % accuracy of household problem
timefactor = options.timefactor; % increase to allow for more time solving the household problem

load(policy_name) % load initial guess for policy functions


z_no = length(z); % number of aggregate states
grid_Nno = length(grid_N); % number of grid points on grid for N
x_no = length(x_grid); % numer of grid points for financial wealth

if length(mu) == 1 % if mu is scalar, use same mu in each aggregate state
    mu = mu*ones(z_no,1); 
end


if length(coeffs) == 4*z_no % only linear law used
    coeffs(4*z_no+(1:(2*z_no))) = 0; % set quadratic coefficients 0 and estimate linear law in the end
    lawtype = 'linear';
elseif length(coeffs) == 6*z_no % quadratic law
    lawtype = 'quadratic';
elseif length(coeffs) == 10 % constant log(z) log(N/meanN) log(z)^2 log(N/meanN)^2 for log J and log P
    lawtype = 'logquadratic';
end



% Aggregate states summarized in S: {p,N}, where p is the number of the
% productivity state and N is the value of aggregate employment
% Compute some aggregate variables as functions of aggregate state:
if strcmp(lawtype,'logquadratic')
    aggJ = @(S) exp(coeffs(1) + coeffs(2)*log(z(S{1})) + coeffs(3)*log(S{2}/meanN) + coeffs(4)*log(z(S{1})).^2 + coeffs(5)*log(S{2}/meanN).^2); % real price of equity
    aggP = @(S) exp(coeffs(6) + coeffs(7)*log(z(S{1})) + coeffs(8)*log(S{2}/meanN) + coeffs(9)*log(z(S{1})).^2 + coeffs(10)*log(S{2}/meanN).^2); % price level
else % quadratic law
    aggJ = @(S) exp(coeffs(S{1})       +log(S{2}/meanN).*coeffs(z_no+S{1})  +log(S{2}/meanN).^2.*coeffs(4*z_no+S{1})); % real price of equity
    aggP = @(S) exp(coeffs(2*z_no+S{1})+log(S{2}/meanN).*coeffs(3*z_no+S{1})+log(S{2}/meanN).^2.*coeffs(5*z_no+S{1})); % price level
end
    
agglambdaf = @(S) psi*aggJ(S).^(-1); % filling probability of vacancies
aggtheta   = @(S) (phi0./agglambdaf(S)).^(1/(1-phi1)); % labor market tightness
agglambdaw = @(S) agglambdaf(S).*aggtheta(S); % transition probability for unemployed
aggNp = @(S) (1-delta)*S{2}+agglambdaw(S).*(1-S{2}); % employment next period
aggPnext = @(k,S) aggP({k,aggNp(S)}); % P' as a function of productivity state next period, k


% Find long run values of employment in each state
Nlimit = zeros(1,z_no);
solver_options = optimset('Display','off','tolFun',1e-12,'tolX',1e-12);
for i=1:z_no
   Nlimit(i)=fsolve(@(x)aggNp({i,x})-x,0.9,solver_options); % long run value 
end
% find unconditional distribution over states in z
[tempv,~]=eigs(z_prob',1);
tempv=tempv/sum(tempv);

% average P is defined as the long-run value of P in each state weighted with the unconditional distribution
meanP = aggP({1:z_no,Nlimit})*tempv;

% average z is defined as the exponential of unconditional mean of log(z)
meanz = exp(log(z(1:z_no))*tempv);


% Interest rate rule
aggPb = @(S) 1./max(IR_coeff(3),IR_coeff(1)*(aggP(S)/meanP).^IR_coeff(2)); % nominal price of bond = 1/(1+i)


aggwage = @(S) w_coeff(1) * (z(S{1})/meanz).^w_coeff(2) .* (aggP(S)/meanP).^(w_coeff(3)-1); % real wage
aggd = @(S) z(S{1})-aggwage(S); % real dividends

% after tax real income of employed (aggregate income with balanced budget = w*N - (1-Pb)*B/P)
aggincome = @(S) (aggwage(S).*S{2}-(1-aggPb(S)).*B./aggP(S))./(S{2}+(1-S{2}).*mu(S{1})); 

aggJdnext = @(k,S) aggd({k,aggNp(S)})+aggJ({k,aggNp(S)})*(1-delta); % cum-dividend real value of shares next period in state k




% calculate aggregate matrices that are needed in the household problem
% matrices used in the current period are 4-dimensional: (z,N,x,eps)
% matrices used in the next period are 6-dimensional: (z',eps',z,N,x,eps)

aggx = repmat(reshape(x_grid,[1,1,x_no,1]),[z_no,grid_Nno,1,2]); % financial wealth

aggJL = repmat(aggJ({repmat((1:z_no)',[1,grid_Nno]),repmat(grid_N,[z_no,1])}),[1,1,x_no,2]); % real price of equity this period
aggdL = repmat(aggd({repmat((1:z_no)',[1,grid_Nno]),repmat(grid_N,[z_no,1])}),[1,1,x_no,2]); % real dividends this period
aggPL = repmat(aggP({repmat((1:z_no)',[1,grid_Nno]),repmat(grid_N,[z_no,1])}),[1,1,x_no,2]); % price level this period
aggPbL  = repmat(aggPb({repmat((1:z_no)',[1,grid_Nno]),repmat(grid_N,[z_no,1])}),[1,1,x_no,2]); % real price of bond this period

aggwL = repmat(aggincome({repmat((1:z_no)',[1,grid_Nno]),repmat(grid_N,[z_no,1])}),[1,1,x_no,2]); % real income this period
for i=1:z_no
    aggwL(i,:,:,2) = mu(i)*aggwL(i,:,:,1); % real income of unemployed
end

tempznext = repmat(reshape(1:z_no,[z_no,1,1]),[1,z_no,grid_Nno]); % z' for all possible combinations of z',z,N
tempz = repmat(reshape(1:z_no,[1,z_no,1]),[z_no,1,grid_Nno]); % z for all possible combinations of z',z,N
tempN = repmat(reshape(grid_N,[1,1,grid_Nno]),[z_no,z_no,1]); % N for all possible combinations of z',z,N
if z_no==1
    tempznext = ones(1,grid_Nno);
    tempz = ones(1,grid_Nno);
    tempN = grid_N;
end


% beginning of period real value of equity next period
aggJdnextL(1:z_no,1,:,:,1,1) = aggJdnext(tempznext,{tempz,tempN});
aggJdnextL = repmat(aggJdnextL,[1,2,1,1,x_no,2]);


% P' next period
aggPnextL(1:z_no,1,:,:,1,1) = aggPnext(tempznext,{tempz,tempN});
aggPnextL = repmat(aggPnextL,[1,2,1,1,x_no,2]);



% job-finding probability
agglambdawL(1,1,:,:,1,1) = agglambdaw({repmat((1:z_no)',1,grid_Nno),repmat(grid_N,z_no,1)});
agglambdawL = repmat(agglambdawL,[z_no,2,1,1,x_no,2]);

% transition probability including aggregate and idiosyncratic transition
aggprobL = ones(z_no,2,z_no,grid_Nno,x_no,2);
% idiosyncratic transition
aggprobL(:,1,:,:,:,1) = aggprobL(:,1,:,:,:,1)*(1-delta); % E-E
aggprobL(:,2,:,:,:,1) = aggprobL(:,2,:,:,:,1)*delta; % E-U
aggprobL(:,1,:,:,:,2) = aggprobL(:,1,:,:,:,2).*agglambdawL(:,1,:,:,:,2); % U-E
aggprobL(:,2,:,:,:,2) = aggprobL(:,2,:,:,:,2).*(1-agglambdawL(:,2,:,:,:,2)); % U-U
% aggregate transition
for i=1:z_no % agg state this period
    for j=1:z_no % agg state next period
        aggprobL(j,:,i,:,:,:) = z_prob(i,j)*aggprobL(j,:,i,:,:,:);
    end
end


% matrices that are needed to interpolate when calculating consumption next
% period in all states (z',eps',z,n,x,eps)
% linear interpolation with respect to N
for i = 1:z_no % productivity this period
    for j = 1:grid_Nno % aggregate employment this period
        tempN_index = min(find(aggNp({i,grid_N(j)})>=grid_N,1,'last'),grid_Nno-1);
        if isempty(tempN_index) %allow for extrapolation
            interpN_index(i,j)=1;
        else
            interpN_index(i,j)=tempN_index;
        end
    end
end
interpN_weight = (aggNp({repmat((1:z_no)',[1,grid_Nno]),repmat(grid_N,[z_no,1])})-grid_N(interpN_index+1))./(grid_N(interpN_index)-grid_N(interpN_index+1)); % weight of the distribution that moves to index
interpN_weight = max(-2,min(3,interpN_weight)); % limit extrapolation
interpN_index = repmat(reshape(interpN_index,[1,1,z_no,grid_Nno,1,1]),[z_no,2,1,1,x_no,2]);
interpN_weight = repmat(reshape(interpN_weight,[1,1,z_no,grid_Nno,1,1]),[z_no,2,1,1,x_no,2]);

interpz_index = repmat(reshape(1:z_no,[z_no,1,1,1,1,1]),[1,2,z_no,grid_Nno,x_no,2]); % next period's productivity
interpeps_index = repmat(reshape([1,2],[1,2,1,1,1,1]),[z_no,1,z_no,grid_Nno,x_no,2]); % next period's employment status

% dampening factor for policy functions
HHdampening = 0.2;

smalleps = 1e-10; % small number to ensure that consumption and bond holdings remain positive while iterating
tic
% Start of loop for household's problem
d2 = 1;
while d2 > HHcrit
    % calculate real financial wealth and consumption next period (states: (z',eps',z,n,x,eps))
    tempq = repmat(reshape(aggq,[1,1,z_no,grid_Nno,x_no,2]),[z_no,2,1,1,1,1]); % equity holdings
    tempb = repmat(reshape(aggb,[1,1,z_no,grid_Nno,x_no,2]),[z_no,2,1,1,1,1]); % bond holdings
    aggxnext = aggJdnextL.*tempq+tempb./aggPnextL; % real financial wealth next period

    % interpolation with respect to cash on hand next period
    temp = interp1(x_grid,1:x_no,aggxnext,'linear','extrap');
    interpx_index = max(1,min(floor(temp),x_no-1));
    interpx_weight = 1+interpx_index-temp;
 
    % consumption next period
    aggcnext = max(smalleps,interpN_weight.*interpx_weight.*    aggc(sub2ind([z_no,grid_Nno,x_no,2],interpz_index,interpN_index,interpx_index,interpeps_index))+...
        (1-interpN_weight).*interpx_weight.*    aggc(sub2ind([z_no,grid_Nno,x_no,2],interpz_index,interpN_index+1,interpx_index,interpeps_index))+...
            interpN_weight.*(1-interpx_weight).*aggc(sub2ind([z_no,grid_Nno,x_no,2],interpz_index,interpN_index,interpx_index+1,interpeps_index))+...
        (1-interpN_weight).*(1-interpx_weight).*aggc(sub2ind([z_no,grid_Nno,x_no,2],interpz_index,interpN_index+1,interpx_index+1,interpeps_index)));

    
    % Marginal utility of consumption next period
    aggMUcnext = aggcnext.^(-gamma);
    
    % Calculate expected marginal utility on the right hand side of FOC for equity
    aggEMUq(1:z_no,:,:,:) = beta*squeeze(sum(sum(aggprobL.*aggMUcnext.*aggJdnextL,1),2))./squeeze(aggJL);

    % Calculate expected marginal utility on the right hand side of FOC for bonds
    aggEMUb(1:z_no,:,:,:) = beta*squeeze(sum(sum(aggprobL.*aggMUcnext./aggPnextL,1),2)).*squeeze(aggPL./aggPbL);
    if chi>0 % add marginual utility of holding bonds
        aggEMUb = aggEMUb + chi*((aggb-b_min)./aggPL.*aggPbL).^(-zeta);
    end
    
    % consumption from FOC that is binding or equal to maximum consumption
    aggc_new = min(aggwL+aggx-aggJL*q_min-aggPbL./aggPL*b_min,max(smalleps,max(aggEMUq,aggEMUb)).^(-1/gamma));
    
    % adjust demand for equity proportional to total amount invested and adjust it 
    % downwards (upwards), if right hand side of FOC for equity (aggEMUq) is smaller
    % (larger) than right hand side of FOC for money (aggEMUb)
    aggq_new = max(q_min,min((aggwL+aggx-aggc_new-aggPbL./aggPL*b_min-chi*smalleps*10^5)./aggJL,q_min+(aggq-q_min).*(aggwL+aggx-aggc_new-aggJL*q_min-aggPbL./aggPL*b_min)./(aggwL+aggx-aggc-aggJL*q_min-aggPbL./aggPL*b_min)+100*log(aggEMUq./aggEMUb)*meanz));
    
    aggb_new = (aggwL+aggx-aggc_new-aggJL.*aggq_new).*aggPL./aggPbL; % demand for bonds from budget constraint
    
    % Calculate differences between new and old policy functions
    d1_c = abs(aggc_new-aggc)./(1+aggc);
    d1_c_max = max(d1_c(:));
    d1_b = abs(aggb_new-aggb)./(1+aggb);
    d1_b_max = max(d1_b(:));
    d1_q = abs(aggq_new-aggq)./(1+aggq);
    d1_q_max = max(d1_q(:));
    d2 = max([d1_c_max,d1_b_max,d1_q_max]);

    % remove comment to draw deviation of policy function while iterating
    %{
    for i = 1:z_no
        for j=1:grid_Nno
            subplot(z_no,grid_Nno,(i-1)*grid_Nno+j)
            plot(x_grid,[squeeze(aggq(i,j,:,:)-aggq_new(i,j,:,:)),squeeze(aggb(i,j,:,:)-aggb_new(i,j,:,:)),squeeze(aggc(i,j,:,:)-aggc_new(i,j,:,:))])
            title(num2str(max([max(max(abs(aggc_new(i,j,:,:)-aggc(i,j,:,:))./(1+aggc(i,j,:,:)))),max(max(abs(aggq_new(i,j,:,:)-aggq(i,j,:,:))./(1+aggq(i,j,:,:)))),max(max(abs(aggb_new(i,j,:,:)-aggb(i,j,:,:))./(1+aggb(i,j,:,:))))])));
            xlim([0,x_grid(end-1)*1.1])
        end
    end
    drawnow 
    %}

    % update policy functions
    dampening = 100/(100+max(0,toc/timefactor-5)^2)*HHdampening; % dampen more after longer running time
    aggc = aggc+dampening*(aggc_new-aggc);
    aggb = aggb+dampening*(aggb_new-aggb);
    aggq = aggq+dampening*(aggq_new-aggq);
 
    if toc>120*timefactor % stop iterating, if it hasn't converged by that time
        disp(['Household problem: time out, error: ',num2str(d2)])
        break
    end

end

% Check whether constraint that was imposed in household problem to ensure nonnegative future consumption is binding
if min(aggcnext(:)<= 2*smalleps)
    disp('HH problem, future consumption binding')
end

% Check whether nonnegativity of MRS is satisfied
test = max(aggEMUq,aggEMUb);
if min(test(:)<= 2*smalleps)
    disp('HH problem, nonnegativity of MRS binding')
end


% Check whether borrowing constraint for equity is satisfied
if min(aggq(:)<= q_min -smalleps)
    disp('HH problem, borrowing constraint equity not satisfied')
end
aggq = max(q_min,aggq);

% Check whether borrowing constraint for bonds is satisfied
if min(aggb(:)<= b_min -smalleps)
    disp('HH problem, borrowing constraint bonds not satisfied')
end
aggb = max(b_min,aggb);

% Check whether budget constraint is satisfied
testBC = aggwL+aggx-aggc-aggJL.*aggq-aggb./aggPL.*aggPbL;
if min(abs(testBC(:))>= smalleps)
    disp(['HH problem, budget constraint not satisfied, max deviation: ',num2str(max(abs(testBC(:))))])
end
aggc = aggwL+aggx-aggJL.*aggq-aggb./aggPL.*aggPbL;

if chi>0
    test_bonds = aggwL+aggx-aggc_new-aggPbL./aggPL*b_min - aggq.*aggJL;
    if min(test_bonds(:))< chi*smalleps*10^5+smalleps
        disp('HH problem, minimum bond holdings binding')
    end
end


% check for extrapolation on grid for financial wealth to compare after
% simulation if extrapolation in occurs in the ergodic distribution
aggindex =  repmat(reshape(1:x_no,[1,1,x_no,1]),[z_no,grid_Nno,1,2]); % matrix with indices of financial wealth
test_extrapL = squeeze(aggindex).*squeeze((max(max(interpx_weight,[],1),[],2)>(1+smalleps))); % indices of financial wealth grid if interpolated at low end
test_extrapH = squeeze(aggindex).*squeeze((min(min(interpx_weight,[],1),[],2)<(-smalleps))); % indices of financial wealth if interpolated at high end
test_extrapH(test_extrapH==0)=inf;
for i=1:z_no
    % boundaries of grid for financial wealth such that in all possible
    % states next period there is no extrapolation needed
    extrap_bounds(i,:) = x_grid([max(1,max(max(max(test_extrapL(i,:,:,:))))),min(x_no,min(min(min(test_extrapH(i,:,:,:))))) ]);
end


% draw policy functions
% remove comment to draw convergedpolicy functions
%{
for i = 1:z_no
    for j=1:grid_Nno
        subplot(z_no,grid_Nno,(i-1)*grid_Nno+j)
        plot(x_grid,[squeeze(aggq(i,j,:,:)),squeeze(aggb(i,j,:,:)),squeeze(aggc(i,j,:,:))])
        title(num2str(max([max(max(abs(aggc_new(i,j,:,:)-aggc(i,j,:,:))./(1+aggc(i,j,:,:)))),max(max(abs(aggq_new(i,j,:,:)-aggq(i,j,:,:))./(1+aggq(i,j,:,:)))),max(max(abs(aggb_new(i,j,:,:)-aggb(i,j,:,:))./(1+aggb(i,j,:,:))))])));
    end
end
drawnow
%}





if nargout>2 % provide additional statistics for sim_details
    
% Marginal rate of substitution beta*E(c'/c)^-gamma
aggstatistics.MRS(1:z_no,:,:,:) = beta*squeeze(sum(sum(aggprobL.*aggMUcnext,1),2)).*squeeze(aggc).^gamma;

% Term needed for shadow nominal interest rate (accounting for money in the utility part)
aggstatistics.X(1:z_no,:,:,:) = (squeeze(sum(sum(aggprobL.*aggMUcnext./aggPnextL,1),2)).*squeeze(aggPL)).^(-1/zeta); % (E[c'^-gamma*P/P'])^(1-zeta)

% Calculate expected life-time utility    
currentU = (aggc.^(1-gamma)-1)/(1-gamma); % utility from consumption this period
if chi>0 % adding utility from holding bonds this period, if applicable
    currentU = currentU+chi*(((aggb-b_min)./aggPL.*aggPbL).^(1-zeta)-1)/(1-zeta); 
end

aggU = currentU/(1-beta); % initial guess for value function

d3=1;
while d3 > 1e-10
    % calculate value function next period
    aggUnext =    interpN_weight.*interpx_weight.*     aggU(sub2ind([z_no,grid_Nno,x_no,2],interpz_index,interpN_index,interpx_index,interpeps_index))+...
            (1-interpN_weight).*interpx_weight.*    aggU(sub2ind([z_no,grid_Nno,x_no,2],interpz_index,interpN_index+1,interpx_index,interpeps_index))+...
                interpN_weight.*(1-interpx_weight).*aggU(sub2ind([z_no,grid_Nno,x_no,2],interpz_index,interpN_index,interpx_index+1,interpeps_index))+...
            (1-interpN_weight).*(1-interpx_weight).*aggU(sub2ind([z_no,grid_Nno,x_no,2],interpz_index,interpN_index+1,interpx_index+1,interpeps_index));

    % updated value function this period
    aggUnew(1:z_no,:,:,:) = squeeze(currentU) + beta*squeeze(sum(sum(aggprobL.*aggUnext,1),2));
    d3 = max(abs((aggUnew(:)-aggU(:))./(1+abs(aggU(:)))));
    aggU = aggUnew;
end

aggstatistics.utility = aggU;


% calculate implied bargaining power of worker = (U(emp)-U(unemp))/(U(emp)-U(unemp)+(1-tax)*U'(emp)*(J*(1-delta)+d)))

tempworker = squeeze(aggU(:,:,:,1)-aggU(:,:,:,2)); % worker surplus
tempfirm = squeeze(aggJL(:,:,:,1)*(1-delta)+aggdL(:,:,:,1)); % firm surplus

temptax = 1-aggwL./repmat(aggwage({repmat((1:z_no)',[1,grid_Nno]),repmat(grid_N,[z_no,1])}),[1,1,x_no,2]); % tax rate this period
temptax = squeeze(temptax(:,:,:,1));

% find derivative of value function of worker with respect to financial wealth
tempderiv(1:z_no,:,1) =  squeeze((aggU(:,:,2,1)-aggU(:,:,1,1))/(x_grid(2)-x_grid(1)));
tempderiv(1:z_no,:,x_no) =  squeeze((aggU(:,:,x_no,1)-aggU(:,:,x_no-1,1))/(x_grid(x_no)-x_grid(x_no-1)));
for i=2:(x_no-1)
    tempderiv(1:z_no,:,i) = squeeze(0.5*(aggU(:,:,i+1,1)-aggU(:,:,i,1))/(x_grid(i+1)-x_grid(i))+0.5*(aggU(:,:,i,1)-aggU(:,:,i-1,1))/(x_grid(i)-x_grid(i-1)))  ;
end

aggstatistics.bpower(1:z_no,:,:) = tempworker./(tempworker+(1-temptax).*squeeze(tempderiv).*tempfirm);


aggstatistics.names4 = {'MRS','utility','X'}; % names of variables that depend on (z,N,x,eps)
aggstatistics.names3 = {'bpower'}; % names of variables that depend on (z,N,x) (only calculated for employed)

end



% Simulate the economy using the histogram method
rng default % set seed for random number generator (only needed for transition of individual agent's simulation)

init_q = options.init_q; % initial allocation of equity in the histogram of agents
init_b = options.init_b; % initial allocation of bonds in the histogram of agents
init_dist = options.init_dist; % initial distribution of agents at each grid point of the histogram
hist_grid = options.hist_grid; % grid for histogram of agents used to simulate the economy
hist_no = length(hist_grid);  % number of grid points for histogram
sim_p = options.sim_p; % series of aggregate productivity states used for simulation
sim_no = options.sim_no; % number of periods simulated
sim_drop = options.sim_drop; % number of initial periods dropped

sim_z = reshape(z(sim_p),sim_no,1); % productivity


% Find long series of remaining in the same productivity state
t_longrun = ones(1,z_no); 
if z_no>1
    for i=1:z_no
        temp_avgdur = round(1/(1-z_prob(i,i))); % average duration of staying in that state
        for k=(sim_drop+temp_avgdur):sim_no
            temp = sim_p(k+1-(1:temp_avgdur))==i; % check whether last periods where all in that state
            if min(temp) == 1
                t_longrun(i) = k; % first period after average duration in that state
                break
            end
            if k==sim_no && sim_no>100+sim_drop+temp_avgdur
               disp(['Warning: Not enough consecutive periods in state ',num2str(i),' to calculate long run value'])
            end
        end
    end
else
    t_longrun(1) = sim_no;
end


% interpolate policy functions for finer grid, on which distribution is simulated
for i = 1:z_no 
    for j=1:grid_Nno
        for k = 1:2
            aggq_hist(i,j,:,k) = interp1(x_grid,squeeze(aggq(i,j,:,k)),hist_grid,'linear','extrap');
            aggb_hist(i,j,:,k) = interp1(x_grid,squeeze(aggb(i,j,:,k)),hist_grid,'linear','extrap');
            aggc_hist(i,j,:,k) = interp1(x_grid,squeeze(aggc(i,j,:,k)),hist_grid,'linear','extrap');
            if nargout > 2
                for i_name = 1:length(aggstatistics.names4)
                    aggstatistics_hist.(aggstatistics.names4{i_name})(i,j,:,k) = interp1(x_grid,squeeze(aggstatistics.(aggstatistics.names4{i_name})(i,j,:,k)),hist_grid,'linear','extrap');
                end
            end
            
        end
        
        if nargout > 2
            for i_name = 1:length(aggstatistics.names3)
                aggstatistics_hist.(aggstatistics.names3{i_name})(i,j,:) = interp1(x_grid,squeeze(aggstatistics.(aggstatistics.names3{i_name})(i,j,:)),hist_grid,'linear','extrap');
            end
            
        end
    end
end

sim_q = init_q; % last period's true allocation of q
sim_b = init_b; % last period's true allocation of nominal m
sim_dist = init_dist; % last period's histogram of perceived cash on hand


sim_N=zeros(sim_no,1);
sim_J=zeros(sim_no,1);
sim_P=zeros(sim_no,1);
sim_Jp=zeros(sim_no,1);
sim_Pp=zeros(sim_no,1);
sim_Pbp=zeros(sim_no,1);

sim_w=zeros(sim_no,1);
sim_Erb=zeros(sim_no,1);
sim_Erq=zeros(sim_no,1);
sim_Einf=zeros(sim_no,1);

sim_tax=zeros(sim_no,1);
sim_dp=zeros(sim_no,1);
sim_Bp=zeros(sim_no,1);
sim_Q=zeros(sim_no,1);
sim_Qp=zeros(sim_no,1);
sim_Cp=zeros(sim_no,1);
sim_theta=zeros(sim_no,1);
sim_income=zeros(sim_no,2);

sim_wealthbounds = repmat([+inf,-inf],z_no,1); % simulated boundaries of wealth distribution, conditional on productivity
hist_grid2 = repmat(hist_grid',1,2); % replicate financial wealth grid for both employment types

if nargout > 2
    % Variable needed for simulation of time paths of agents
    sim_individual_no = options.sim_individual_no; % number of agents when simulating them separately 
    ind.emp = zeros(sim_no,sim_individual_no); % employment status
    ind.unempduration = zeros(sim_no,sim_individual_no); % duration of current unemployment spell
    ind.fw = zeros(sim_no,sim_individual_no); % realized real financial wealth
    ind.c = zeros(sim_no,sim_individual_no); % realized consumption
    ind.b = zeros(sim_no,sim_individual_no); % realized bond holdings
    ind.q = zeros(sim_no,sim_individual_no); % realized equity holdings
    ind.fwp = zeros(sim_no,sim_individual_no); % perceived real financial wealth
    ind.income = zeros(sim_no,sim_individual_no); % realized after tax income
    ind.cp = zeros(sim_no,sim_individual_no); % perceived consumption
    ind.bp = zeros(sim_no,sim_individual_no); % perceived bond holdings
    ind.qp = zeros(sim_no,sim_individual_no); % perceived equity holdings
    ind.bpower = zeros(sim_no,sim_individual_no); % bargaining power
    ind.deposits2income = zeros(sim_no,sim_individual_no); % value of bond holdings to annual income
    ind.assets2income = zeros(sim_no,sim_individual_no); % financial wealth to annual income
    
    % draw indviduals from initial distribution
    temp = min((1+sum(repmat(rand(1,sim_individual_no),2*hist_no,1)>=repmat(cumsum([sim_dist(:,1);sim_dist(:,2)]),1,sim_individual_no))),2*hist_no);
    ind.lastemp = 1+floor((temp-1)/hist_no); % initial employment (1=employed, 2=unemployed)
    ind.lastq = init_q(temp); % initial equity holdings
    ind.lastb = init_b(temp); % initial bond holdings
end


for t=1:sim_no % Simulate
    sim_N(t) = sum(sum(sim_dist.*sim_q)); % employment
    
    sim_N_index = min(find(sim_N(t)>=grid_N,1,'last'),grid_Nno-1); % linear interpolation with respect to N
    if isempty(sim_N_index) %allow for extrapolation
        sim_N_index=1;
    end
    sim_N_weight = (sim_N(t)-grid_N(sim_N_index+1))/(grid_N(sim_N_index)-grid_N(sim_N_index+1)); % weight of the distribution that moves to index
    sim_N_weight = max(-2,min(3,sim_N_weight)); % limit extrapolation

    % Interpolate policy functions with respect to aggregate employment
    sim_qp = max(q_min,squeeze(sim_N_weight*aggq_hist(sim_p(t),sim_N_index,:,:)+(1-sim_N_weight)*aggq_hist(sim_p(t),sim_N_index+1,:,:))); % individual demand for equity
    sim_bp = max(b_min,squeeze(sim_N_weight*aggb_hist(sim_p(t),sim_N_index,:,:)+(1-sim_N_weight)*aggb_hist(sim_p(t),sim_N_index+1,:,:))); % individual demand for nominal bonds
    sim_cp =            squeeze(sim_N_weight*aggc_hist(sim_p(t),sim_N_index,:,:)+(1-sim_N_weight)*aggc_hist(sim_p(t),sim_N_index+1,:,:)); % individual demand for consumption
    
    
    sim_states = {sim_p(t),sim_N(t)}; % aggregate state variables (productivity state, employment)
    sim_Jp(t) = aggJ(sim_states); % perceived J
    sim_Pp(t) = aggP(sim_states); % perceived P
    sim_Pbp(t) = aggPb(sim_states); % perceived bond price
    sim_w(t) = aggwage(sim_states); % real wage
   
    sim_tax(t) = 1-(sim_N(t)*sim_w(t)-(1-sim_Pbp(t))*B/sim_Pp(t))/((1-sim_N(t))*mu(sim_p(t))+sim_N(t))/sim_w(t); % tax rate
    sim_income(t,:) = (1-sim_tax(t))*sim_w(t)*[1,mu(sim_p(t))]; % after tax income
    sim_dp(t) = sim_z(t)-sim_w(t); % perceived real dividend

    sim_Erq(t) = z_prob(sim_p(t),:)*aggJdnext(1:z_no,sim_states)'/sim_Jp(t)-1; % expected return on equity
    sim_Einf(t) = z_prob(sim_p(t),:)*(aggPnext(1:z_no,sim_states)'/sim_Pp(t))-1; % expected inflation rate
    sim_Erb(t) = z_prob(sim_p(t),:)*(sim_Pp(t)./aggPnext(1:z_no,sim_states)')/sim_Pbp(t)-1; % expected real return on bonds
    
    for i=1:2 % current employment status
        % perceived financial wealth conditional on last period's and this period's employment
        sim_xp(i,:,:) = ((1-delta)*sim_Jp(t)+sim_dp(t))*sim_q+sim_b/sim_Pp(t);
    end
    
    % transition of histogram (including transition in labour market)
    sim_lambdaw = (sim_N(t)-(1-delta)*sum(sim_dist(:,1)))/sum(sim_dist(:,2)); % job-finding probability implied by current employment and employment last period
    sim_transition = [1-delta,delta;sim_lambdaw,1-sim_lambdaw]; % transition probabilities 
    
    % interpolation of financial wealth
    tempindex = interp1(hist_grid,1:hist_no,sim_xp,'linear','extrap');
    hist_index = max(1,min(floor(tempindex),hist_no-1));
    hist_weight = max(0,min(1,(sim_xp-hist_grid(hist_index+1))./(hist_grid(hist_index)-hist_grid(hist_index+1)))); % weight of the distribution that moves to index
    
    sim_dist_new = zeros(hist_no,2); % distribution in the beginning of the period
    % add weight for each combination of this period's employment, last
    % period's employment, and last period's financial wealth
    for i=1:2 % employment this period
        for j=1:2 % employment last period
            for k=1:hist_no % grid point of financial wealth last period
                sim_dist_new(hist_index(i,k,j),i) =   sim_dist_new(hist_index(i,k,j),i) + hist_weight(i,k,j)*sim_dist(k,j)*sim_transition(j,i);
                sim_dist_new(hist_index(i,k,j)+1,i) = sim_dist_new(hist_index(i,k,j)+1,i) + (1-hist_weight(i,k,j))*sim_dist(k,j)*sim_transition(j,i);
            end
        end
    end
    

    sim_wealthbounds(sim_p(t),1) = min(sim_wealthbounds(sim_p(t),1),min(min(hist_grid2(cumsum(abs(sim_dist_new))>1e-6)))); % find minimum value of wealth observed in simulation
    sim_wealthbounds(sim_p(t),2) = max(sim_wealthbounds(sim_p(t),2),max(max(hist_grid2(cumsum(abs(sim_dist_new))<repmat(sum(abs(sim_dist_new))-1e-6,hist_no,1))))); % find maximum value of wealth observed in simulation

    sim_Bp(t) = sum(sum(sim_bp.*sim_dist_new)); % aggregate demand for bonds
    sim_Qp(t) = sum(sum(sim_qp.*sim_dist_new)); % aggregate demand for equity
    sim_Cp(t) = sum(sum(sim_cp.*sim_dist_new)); % aggregate demand for consumption
    
    % market clearing in the bond market
    if sim_Bp(t)>B/1000
        sim_P(t) = ((B-b_min)/(sim_Bp(t)-b_min))^(1/elas)*sim_Pp(t);    % market clearing price
        sim_b = b_min+max(0,(sim_bp-b_min)*(sim_P(t)/sim_Pp(t))^elas);       % allocation of bonds
    else
        % if there is no demand for bonds, distribute supply evenly (only happens, if aggregate laws are too far off)
        sim_P(t)=10*sim_Pp(t);
        sim_b = B*ones(hist_no,2);
    end
        
    try
        % market clearing price in the equity market: aggregate supply
        % given from matching function must be equal to aggregate demand
        if elas==1 && phi1 == 0.5 % analytical solution for J possible
            sim_J(t) = (-(1-delta)*sim_N(t)+sqrt(((1-delta)*sim_N(t))^2+4*phi0^2/psi*(1-sim_N(t))*sim_Qp(t)*sim_Jp(t)))/(2*phi0^2/psi*(1-sim_N(t)));
        else
            solver_options = optimset('Display','off','tolFun',1e-12,'tolX',1e-12);
            sim_J(t) = fsolve(@(J)(1-delta)*sim_N(t)+phi0*(psi/phi0/J).^(phi1/(phi1-1))*(1-sim_N(t))-sim_Qp(t)*(sim_Jp(t)/J)^elas,sim_Jp(t),solver_options);            
        end
    catch
        sim_J(t) = 0;
    end
    if sim_J(t)>0.01 && sim_Qp(t)>0.01
        sim_Q(t) = min(0.999,max(sim_N(t)*(1-delta),(sim_Jp(t)/sim_J(t))^elas*sim_Qp(t)));
        sim_q = q_min+max(0,(sim_qp-q_min)*(sim_Q(t)-q_min)/(sim_Qp(t)-q_min));
    else
        % if there is no demand for equity, reduce supply and distribute evenly (only happens, if aggregate laws are too far off)
        sim_J(t) = sim_Jp(t)*0.9;
        sim_Q(t) = max(0.1,min(0.99,sim_N(t)*(1-delta)));
        sim_q = max(q_min,sim_Q(t)*ones(hist_no,2));
    end

    sim_theta(t) = (sim_J(t)/psi*phi0)^(1/(1-phi1)); % labor market tightness

    if nargout > 2 % calculate additional statistics
        for i_name = 1:length(aggstatistics.names4) % average across all agents (e.g. utility)
            sim_details.(aggstatistics.names4{i_name})(t) = sum(sum(sim_dist_new.*squeeze(sim_N_weight*aggstatistics_hist.(aggstatistics.names4{i_name})(sim_p(t),sim_N_index,:,:)+(1-sim_N_weight)*aggstatistics_hist.(aggstatistics.names4{i_name})(sim_p(t),sim_N_index+1,:,:))));
        end
        for i_name = 1:length(aggstatistics.names3) % average across employed (e.g. bargaining power)
            sim_details.(aggstatistics.names3{i_name})(t) = sum(sim_dist_new(:,1).*squeeze(sim_N_weight*aggstatistics_hist.(aggstatistics.names3{i_name})(sim_p(t),sim_N_index,:)+(1-sim_N_weight)*aggstatistics_hist.(aggstatistics.names3{i_name})(sim_p(t),sim_N_index+1,:)))/sum(sim_dist_new(:,1));
        end

        
        % simulated time series of individual agents
        ind.emp(t,:) = 1+(rand(sim_individual_no,1)>sim_transition(ind.lastemp,1)); % new employment status
        if t>1
            ind.unempduration(t,:) = (1+ind.unempduration(t-1,:)).*(ind.emp(t,:)==2); % add 1 to duration of current unemployment spell if still unemployed, 0 if employed
        else
            ind.unempduration(t,:) = (ind.emp(t,:)==2); % first period, as if everyone is newly unemployed
        end
        
        ind.fwp(t,:) = ind.lastb/sim_Pp(t)+(sim_Jp(t)*(1-delta)+sim_dp(t))*ind.lastq; % perceived real financial wealth
        for j=1:2 % calculate policy functions, depending on employment status
            ind.cp(t,ind.emp(t,:)==j) =            interp1(hist_grid,sim_cp(:,j),ind.fwp(t,ind.emp(t,:)==j),'linear','extrap');  % perceived consumption
            ind.bp(t,ind.emp(t,:)==j) = max(b_min,interp1(hist_grid,sim_bp(:,j),ind.fwp(t,ind.emp(t,:)==j),'linear','extrap'));  % perceived nominal bonds
            ind.qp(t,ind.emp(t,:)==j) = max(q_min,interp1(hist_grid,sim_qp(:,j),ind.fwp(t,ind.emp(t,:)==j),'linear','extrap'));  % perceived equity
        end

        temptax = 1-(sim_N(t)*sim_w(t)-(1-sim_Pbp(t))*B/sim_P(t))/((1-sim_N(t))*mu(sim_p(t))+sim_N(t))/sim_w(t); % realized tax rate (taking into account market-clearing P)
        tempincome= (1-temptax)*sim_w(t)*[1,mu(sim_p(t))]; % realized after tax income
        ind.income(t,:) = tempincome(ind.emp(t,:));
        ind.fw(t,:) = ind.lastb/sim_P(t)+(sim_J(t)*(1-delta)+sim_dp(t))*ind.lastq; % realized financial wealth (taking into account market-clearing P and J)
        ind.b(t,:) = b_min+max(0,(ind.bp(t,:)-b_min)*(sim_P(t)/sim_Pp(t))^elas); % realized bond holdings
        ind.q(t,:) = q_min+max(0,(ind.qp(t,:)-q_min)*(sim_Q(t)-q_min)/(sim_Qp(t)-q_min)); % realized equity holdings
        ind.c(t,:) = max(smalleps,ind.income(t,:)+ind.fw(t,:) -ind.b(t,:)*sim_Pbp(t)/sim_P(t)-ind.q(t,:)*sim_J(t)); % realized consumption from budget constraint
        
        % bargaining power
        ind.bpower(t,ind.emp(t,:)==2) = NaN; % NaN for unemployed
        ind.bpower(t,ind.emp(t,:)==1) = interp1(hist_grid,squeeze(sim_N_weight*aggstatistics_hist.bpower(sim_p(t),sim_N_index,:)+(1-sim_N_weight)*aggstatistics_hist.bpower(sim_p(t),sim_N_index+1,:)),ind.fwp(t,ind.emp(t,:)==1),'linear','extrap'); % bargaining power of employed
        
        ind.deposits2income(t,:) = ind.lastb/sim_P(t)./(4*ind.income(t,:)); % beginning of period value of bond holdings to annualized labour (UI) income
        ind.assets2income(t,:) = ind.fw(t,:)./(4*ind.income(t,:)); % beginning of period financial wealth to annualized labour (UI) income
        
        ind.lastemp = ind.emp(t,:);
        ind.lastb = ind.b(t,:);
        ind.lastq = ind.q(t,:);

    end
        
    if max(t==t_longrun)
        % store distribution and demand for assets after long period in each productivity state
        store_dist{sim_p(t)} = sim_dist_new;
        store_sim_q{sim_p(t)} = sim_q;
        store_sim_b{sim_p(t)} = sim_b;
    end
    
    sim_dist = sim_dist_new; % update distribution
end % end of simulation

sim_C = sim_z.*sim_N-psi*sim_theta.*(1-sim_N); % aggregate consumption

sim_results=[sim_z,sim_N,sim_J,sim_Jp,sim_P,sim_Pp,sim_C,sim_w,sim_tax,sim_Pbp,sim_dp,sim_theta,sim_Erb,sim_Erq,sim_Einf];


if nargout>2 % calculate additional statistics for sim_details
    sim_details.ind_simulation = ind; % store time series of agents

  
    % find observations of 4th period in unemployment spell after at least
    % 8 quarters of employment
    tempdummy4 = zeros(sim_no,sim_individual_no);
    tempcmean11_4 = zeros(sim_no,sim_individual_no);
    tempcmean3_0 = zeros(sim_no,sim_individual_no);
    for t=(sim_drop+12):sim_no
         tempdummy4(t,:) = (ind.emp(t,:)==2) & (ind.emp(t-1,:)==2) & (ind.emp(t-2,:)==2) & (ind.emp(t-3,:)==2) & (ind.emp(t-4,:)==1)  & (ind.emp(t-5,:)==1) ...
             & (ind.emp(t-6,:)==1)  & (ind.emp(t-7,:)==1)  & (ind.emp(t-8,:)==1)  & (ind.emp(t-9,:)==1)  & (ind.emp(t-10,:)==1)  & (ind.emp(t-11,:)==1);
         tempcmean11_4(t,:) = mean(ind.cp((t-11):(t-4),:)); % average consumption in 8 quarters of employment ending 4 quarters before
         tempcmean3_0(t,:) = mean(ind.cp((t-3):(t),:)); % average consumption in last 4 quarters
    end
    tempplagged(5:sim_no,:) = repmat(sim_p(1:(sim_no-4)),1,sim_individual_no); %  productivity state 4 periods before
    
    tempduration = ind.unempduration((sim_drop+1):sim_no,:); % simulated unemployment duration that is not dropped
    tempN = sim_N((sim_drop+1):sim_no); % simulated employment that is not dropped
    tempp = sim_p((sim_drop+1):sim_no); % simulated productivity state that is not dropped
    temppL = repmat(tempp,sim_individual_no,1); % prodcutivity replicated for number of individuals
    tempassets2income = reshape(ind.assets2income((sim_drop+1):sim_no,:),(sim_no-sim_drop)*sim_individual_no,1); % simulated assets to income ratio that is not dropped, as column vector
    
    for i=1:z_no % calculate statistics conditional on productivity
        sim_details.c_drop4(i) = mean(mean(tempcmean3_0(tempplagged==i & tempdummy4==1)./tempcmean11_4(tempplagged==i & tempdummy4==1)-1)); % average consumption drop
        sim_details.meanN(i) = mean(tempN(tempp==i)); % employment
        sim_details.assets2income(i) = median(tempassets2income(temppL==i)); % median financial assets to labour income
        sim_details.unempduration(i) =  mean(tempduration(tempduration>0 & repmat(tempp,1,sim_individual_no)==i)); % average unemployment duration of all observed unemployed
    end
    % unconditional averages
    sim_details.c_drop4(z_no+1) = mean(mean(tempcmean3_0(tempdummy4==1)./tempcmean11_4(tempdummy4==1)-1)); % average consumption drop
    sim_details.meanN(z_no+1) = mean(tempN); % employment
    sim_details.assets2income(z_no+1) = median(tempassets2income); % median financial assets to labour income
    sim_details.unempduration(z_no+1) = mean(tempduration(tempduration>0)); % average unemployment duration of all observed unemployed
    
    sim_details.store_dist = store_dist;
    sim_details.store_sim_q = store_sim_q;
    sim_details.store_sim_b = store_sim_b;
    
    % calculate standard deviations of filtered log variables
    tempvars = [sim_z.*sim_N,sim_N]; % output and employment
    [~,tempcycl] = hpfilter(log(tempvars((sim_drop+1):end,:)),1600);
    sim_details.N_y_volatility = std(tempcycl(:,2))/std(tempcycl(:,1)); % standard deviation of log employment to std of log output
    
    sim_details.SIR = (1./sim_Pbp-1 + chi*(B./(sim_P.*sim_details.X')).^(-zeta))'; % shadow nominal interest rate
    sim_details.t_longrun = t_longrun; % example periods after many periods in each productivity state
    
    % Calculate impulse-response functions
    if z_no==2
        for i=1:z_no % approximate SIR and expected inflation rate with quadratic law conditional on productivity state
            % Calculate shadow nominal interest rate
            tempN = log(sim_N(sim_p==i)); % log employment
            tempSIR = sim_details.SIR(sim_p==i)'; % nominal shadow interest rate
            SIR_coeff(i,:)=  regress(tempSIR,[ones(length(tempN),1),tempN,tempN.^2]); 
            
            % Calculate expected inflation
            for j=1:grid_Nno
                tempEPp(1,j) = (1./aggPnext(1:z_no,{ones(1,z_no)*i,ones(1,z_no)*grid_N(j)}))*z_prob(i,:)'; % expectation of 1/P next period on grid for (z,N)
            end
            tempEPPnext(1,:) = aggP({i,grid_N}).*tempEPp; % expected P/P'
            aggEPPnext_coeff(i,:) = regress(log(tempEPPnext(1,:))',[ones(grid_Nno,1),log(grid_N)',log(grid_N).^2']); % approximate expected inflation quadratically conditional on z            
        end
        aggSIR = @(S)SIR_coeff(S{1})+SIR_coeff(z_no+S{1}).*log(S{2})+SIR_coeff(2*z_no+S{1}).*log(S{2}).^2;
        aggEPPnext = @(S) exp(aggEPPnext_coeff(S{1})+aggEPPnext_coeff(z_no+S{1}).*log(S{2})+aggEPPnext_coeff(2*z_no+S{1}).*log(S{2}).^2); % expected P/P'

        % interpolate variables on finer grid, used for calculation of
        % IRFs: one row for each value of employment, one column for each
        % value of productivity
        IRF_gridno = 500;
        IRF_N = repmat(linspace(grid_N(1),grid_N(grid_Nno),IRF_gridno)',1,z_no);
        IRF_z = repmat(z,IRF_gridno,1); % productivity
        IRF_p = repmat(1:z_no,IRF_gridno,1); % productivity states
        IRF_Nnext = aggNp({IRF_p,IRF_N}); % employment next period
        IRF_w = aggwage({IRF_p,IRF_N}); % real wage
        IRF_J = aggJ({IRF_p,IRF_N}); % real price of equity
        IRF_P = aggP({IRF_p,IRF_N}); % price level
        IRF_R = 1./aggPb({IRF_p,IRF_N})-1; % nominal interest rate
        IRF_tax = 1-aggincome({IRF_p,IRF_N})./IRF_w; % tax rate
        IRF_taxvol = IRF_tax.*IRF_w.*(IRF_N+repmat(mu',IRF_gridno,1).*(1-IRF_N)); % tax volume
        IRF_Rreal = aggEPPnext({IRF_p,IRF_N})./aggPb({IRF_p,IRF_N})-1; % expected real interest rate
        IRF_SIR = aggSIR({IRF_p,IRF_N}); % shadow nominal interest rate
        IRF_SIRreal = (1+IRF_SIR).*aggEPPnext({IRF_p,IRF_N})-1; % expected real shadow interest rate
        IRF_C = IRF_z.*IRF_N-psi*(1-IRF_N).*aggtheta({IRF_p,IRF_N}); % consumption
        
        IRF_options.IRF_no = 51; % number of periods for IRF
        IRF_options.zstart = [2,1];  % start with good state, then drop to bad state
        IRF_options.percentiles = [0.05,0.95,0.5,0.1,0.9];
        IRF_options.distinit = zeros(IRF_gridno*z_no,1);
        % start at limiting value of employment
        temp = interp1(IRF_N(:,1),1:IRF_gridno,Nlimit(IRF_options.zstart(1)),'linear');
        tempindex = floor(temp);
        tempweight = 1+tempindex-temp;
        IRF_options.distinit((IRF_options.zstart(1)-1)*IRF_gridno+tempindex,1) = tempweight; 
        IRF_options.distinit((IRF_options.zstart(1)-1)*IRF_gridno+tempindex+1,1) = 1-tempweight;
        
        IRF_vars = {IRF_z,IRF_N,IRF_z.*IRF_N,IRF_w,IRF_J,IRF_P,IRF_R,IRF_tax,IRF_taxvol,IRF_Rreal,IRF_w./IRF_z,IRF_SIR,IRF_SIRreal,IRF_C};
        IRF_devs = {'log','ppt','log','log','log','log','ppt','ppt','log','ppt','log','ppt','ppt','log'}; % deviation of variables to be calculated (log or ppt)

        % Compute IRF
        IRF = IRF_calculation(IRF_N(:,1),IRF_Nnext,z_prob,IRF_options,IRF_vars);
        % Compute path without shock
        IRF_options.zstart = [2];
        IRF0 = IRF_calculation(IRF_N(:,1),IRF_Nnext,z_prob,IRF_options,IRF_vars);
        
        for k=1:length(IRF_vars) % calculate deviations
            switch IRF_devs{k}
                case 'log'
                    sim_details.IRF(:,k) = log(IRF.mean{k}./IRF0.mean{k});
                case 'ppt'
                    sim_details.IRF(:,k) = (IRF.mean{k}-IRF0.mean{k})*100;
            end
        end
        
    end
end

% save policy functions
save(policy_name,'grid_N','x_grid','aggb','aggq','aggc')

% Estimate new laws of motion
if strcmp(lawtype,'quadratic') || strcmp(lawtype,'linear')
    % estimate linear or quadratic law, conditional on productivity state
    for i=1:z_no
        temp_indices = sort((sim_p == i).*(1:sim_no)'); % find indices for productivity level
        temp_indices = temp_indices(find(temp_indices>sim_drop,1,'first'):end);

        if(isempty(temp_indices)) % no observation of respective productivity state
            sim_Jcoeff{i}(1:3) = 0;
            sim_Pcoeff{i}(1:3) = 0;
            sim_Jcoeffstats{i} = 0;
            sim_Pcoeffstats{i} = 0;
        elseif rank([ones(length(temp_indices),1),log(sim_N(temp_indices)/meanN)]) == 1 % no variation in employment
            [sim_Jcoeff{i},~,~,~,sim_Jcoeffstats{i}] = regress(log(sim_J(temp_indices)),ones(length(temp_indices),1));
            sim_Jcoeff{i}(2:3) = 0;
            [sim_Pcoeff{i},~,~,~,sim_Pcoeffstats{i}] = regress(log(sim_P(temp_indices)),ones(length(temp_indices),1));
            sim_Pcoeff{i}(2:3) = 0;        
        elseif strcmp(lawtype,'linear') || (rank([ones(length(temp_indices),1),log(sim_N(temp_indices)/meanN),log(sim_N(temp_indices)/meanN).^2]) == 2) % estimate linear law
            [sim_Jcoeff{i},~,~,~,sim_Jcoeffstats{i}] = regress(log(sim_J(temp_indices)),[ones(length(temp_indices),1),log(sim_N(temp_indices)/meanN)]);
            sim_Jcoeff{i}(3) = 0;
            [sim_Pcoeff{i},~,~,~,sim_Pcoeffstats{i}] = regress(log(sim_P(temp_indices)),[ones(length(temp_indices),1),log(sim_N(temp_indices)/meanN)]);
            sim_Pcoeff{i}(3) = 0;
        elseif strcmp(lawtype,'quadratic')  % estimate quadratic law
            [sim_Jcoeff{i},~,~,~,sim_Jcoeffstats{i}] = regress(log(sim_J(temp_indices)),[ones(length(temp_indices),1),log(sim_N(temp_indices)/meanN),log(sim_N(temp_indices)/meanN).^2]);
            [sim_Pcoeff{i},~,~,~,sim_Pcoeffstats{i}] = regress(log(sim_P(temp_indices)),[ones(length(temp_indices),1),log(sim_N(temp_indices)/meanN),log(sim_N(temp_indices)/meanN).^2]);

        end
        tempJR2(i) = sim_Jcoeffstats{i}(1); % R-squared conditional on productivity
        tempPR2(i) = sim_Pcoeffstats{i}(1); % R-squared conditional on productivity
    end
    
    aggJ_new = @(S) exp(sim_Jcoeff{S{1}}(1)+log(S{2}/meanN)*sim_Jcoeff{S{1}}(2)+log(S{2}/meanN).^2*sim_Jcoeff{S{1}}(3)); % newly estimated aggregate law for J
    aggP_new = @(S) exp(sim_Pcoeff{S{1}}(1)+log(S{2}/meanN)*sim_Pcoeff{S{1}}(2)+log(S{2}/meanN).^2*sim_Pcoeff{S{1}}(3)); % newly estimated aggregate law for J


    % Calculate deviation between old and new law of motions
    % for each productivity value calculate log deviation at limiting
    % value of employment of two following productivity states
    % (1+2,2+3,...)
    tempcompN = [Nlimit;Nlimit(2:z_no),Nlimit(z_no-1)];
    if strcmp(lawtype,'linear')
        for i=1:z_no
            tempdev(:,2*i-1) = log(aggJ_new({i,tempcompN(:,i)})./aggJ({i,tempcompN(:,i)}));
            tempdev(:,2*i) =   log(aggP_new({i,tempcompN(:,i)})./aggP({i,tempcompN(:,i)}));
        end
    elseif strcmp(lawtype,'quadratic') % also calculate deviation in the middle of the two points
        for i=1:z_no
            tempdev(:,2*i-1) = log(aggJ_new({i,[tempcompN(:,i);mean(tempcompN(:,i))]})./aggJ({i,[tempcompN(:,i);mean(tempcompN(:,i))]}));
            tempdev(:,2*i) =   log(aggP_new({i,[tempcompN(:,i);mean(tempcompN(:,i))]})./aggP({i,[tempcompN(:,i);mean(tempcompN(:,i))]}));
        end
    end
    dev=tempdev(:);

    
elseif strcmp(lawtype,'logquadratic') % estimate log-quadratic law in employment and productivity
    tempz = log(sim_z((sim_drop+1):sim_no)); % log productivity that is used
    tempN = log(sim_N((sim_drop+1):sim_no)/meanN); % log employment that is used
    tempJ = log(sim_J((sim_drop+1):sim_no)); % log price of equity that is used
    tempP = log(sim_P((sim_drop+1):sim_no)); % log price level that is used
    
    coeffs_new = zeros(1,10);
    [coeffs_new(1:5),~,~,~,sim_Jcoeffstats] = regress(tempJ,[ones(sim_no-sim_drop,1),tempz,tempN,tempz.^2,tempN.^2]);
    [coeffs_new(6:10),~,~,~,sim_Pcoeffstats] = regress(tempP,[ones(sim_no-sim_drop,1),tempz,tempN,tempz.^2,tempN.^2]);
    tempJR2 = sim_Jcoeffstats(1); % unconditional R-squared
    tempPR2 = sim_Pcoeffstats(1); % unconditional R-squared
    
    aggJ_new = @(S) exp(coeffs_new(1) + coeffs_new(2)*log(z(S{1})) + coeffs_new(3)*log(S{2}/meanN) + coeffs_new(4)*log(z(S{1})).^2 + coeffs_new(5)*log(S{2}/meanN).^2);
    aggP_new = @(S) exp(coeffs_new(6) + coeffs_new(7)*log(z(S{1})) + coeffs_new(8)*log(S{2}/meanN) + coeffs_new(9)*log(z(S{1})).^2 + coeffs_new(10)*log(S{2}/meanN).^2);
    
    dev = coeffs_new-coeffs;
end


% remove comment to draw old (perceived) and new (fitted) laws of motion and realized values of J and P
%{
if z_no==2 % set colors for plot
    plot_color = [1 0 0;0 0 1];
else
    plot_color = repmat(1-1./(1:z_no)',1,3);
end

for i=1:z_no
    temp_indices = sort((sim_p == i).*(1:sim_no)'); % find indices for productivity level
    temp_indices = temp_indices(find(temp_indices>sim_drop,1,'first'):end);

   
    % plot price of equity
    subplot(1,2,1)
    if i>1
        hold on
    end
    scatter(sim_N(temp_indices),log(sim_J(temp_indices)),10,plot_color(i,:)) % market clearing values
    hold on
    plot(grid_N,log(aggJ({i,grid_N'})),'Color',plot_color(i,:))  % perceived law
    plot(grid_N,log(aggJ_new({i,grid_N'})),'--','Color',plot_color(i,:)) % fitted law
    hold off
    ylabel('log J')
    xlabel('N')
    
    % plot price level
    subplot(1,2,2)
    if i>1
        hold on
    end
    scatter(sim_N(temp_indices),log(sim_P(temp_indices)),10,plot_color(i,:)) % market clearing values
    hold on
    plot(grid_N,log(aggP({i,grid_N'})),'Color',plot_color(i,:))  % perceived law
    plot(grid_N,log(aggP_new({i,grid_N'})),'--','Color',plot_color(i,:)) % fitted law
    hold off
    ylabel('log P')
    xlabel('N')
    
  
    if i==z_no
        subplot(1,2,1)
        legend('simulated true','perceived','fitted','Location','best')
        title(['R^2: ',num2str(tempJR2)])
        subplot(1,2,2)
        title(['R^2: ',num2str(tempPR2)])
    end
end

drawnow;
%}


% check for extrapolation in terms of aggregate employment
if min(sim_N)<(2*grid_N(1)-grid_N(2))
    disp('########## too much extrapolation w.r.t. employment at lower end ###########')
end
if max(sim_N)>(2*grid_N(grid_Nno)-grid_N(grid_Nno-1))
    disp('########## too much extrapolation w.r.t. employment at upper end ###########')
end


% check whether simulated wealth is inside the interval, in which no
% extrapolation was needed to find the policy functions
if max(sim_wealthbounds(:,1)<extrap_bounds(:,1))
    disp('########## extrapolation at lower tail ###########')
    disp(['simulated lower bounds:            ',num2str(sim_wealthbounds(:,1)')])
    disp(['extrapolation in HH-problem below: ',num2str(extrap_bounds(:,1)')])
end
if max(sim_wealthbounds(:,2)>extrap_bounds(:,2))
    disp('########## extrapolation at upper tail ###########')
    disp(['simulated upper bounds:            ',num2str(sim_wealthbounds(:,2)')])
    disp(['extrapolation in HH-problem above: ',num2str(extrap_bounds(:,2)')])
end

if max(sim_wealthbounds(:,2))>(hist_grid(hist_no)-smalleps)
    % check that distribution doesn't hit upper bound of grid
   disp('########## upper bound of simulated distribution reached #############') 
end

% check that lower bound of grid doesn't exceed borrowing threshold
if (min(q_min*(sim_Jp*(1-delta)+sim_dp) + b_min./sim_Pp)) < hist_grid(1)-smalleps
   disp('########## lower bound of grid above borrowing constraint #############') 
end