function [dev,stats,policy_new] = het_steadystate(params,policy)
% Solves household problem without aggregate uncertainty
%
% INPUTS:
%  params: structure with parameters
%  policy: structure containing
%    x_grid: grid for financial wealth
%    SS_c: guess for policy function for consumption 
%    SS_q: guess for policy function for equity
%    SS_b: guess for policy function for (nominal) bond holdings
%    SS_U: guess for value function
%      (1st column employed, 2nd column unemployed)
%
% OUTPUTS:
%  dev: deviation from market clearing for equity and bonds
%  stats: structure containing
%    SS_J: real price of equity
%    median: structure containing statistics for median employed and
%       unemployed (in terms of financial wealth):
%       assets: real value of assets demanded
%       q: equity demanded
%       b: nominal bonds demanded
%       c: consumption demanded
%    avg: structure containing statistics for average across employed and
%       unemployed:
%       assets: real value of assets demanded
%       q: equity demanded
%       b: nominal bonds demanded
%       c: consumption demanded
%       value: expected life-time utility
%       bpower: bargaining power of employed
%
%  policy_new: structure containing grid for financial wealth and optimal policy functions,
%     refined grid for histogram (hist_grid)m, policy functions on this
%     grid (hist_c, hist_b, hist_q), and distribution of agents (dist)

beta = params.beta; %   discount factor
chi = params.chi; %   scale of bonds in utility (chi*((b-b_min)/P/(1+i))^(-zeta))
zeta = params.zeta; %   curvature of bonds in utitily 
delta = params.delta; %   separation probability
gamma = params.gamma; %   curvature of marginal utility of consumption (c^(-gamma))
B = params.B; %   supply of nominal bonds (each repays 1 unit in t+1)
mu = params.mu; %   unemployment insurance (column vector with mu in each state)
phi0 = params.phi0; %   matching function: phi0*v^phi1*u^(1-phi1)
phi1 = params.phi1; %   matching function: phi0*v^phi1*u^(1-phi1)
psi = params.psi; %   cost of posting vacancy
q_min = params.q_min; %   minimum number of shares that has to be held
b_min = params.b_min; %   minimum number of bonds that has to be held
SS_R = params.IR_coeff(1); %   Interest rate: 1+i_t+1 = IR_coeff(1)
SS_w = params.w_coeff(1); %   real wage: w_coeff(1)
SS_N = params.SS_N; % employment = supply of equity
SS_P = params.SS_P; % price level, needed to calculate outstanding real amount of bonds
z = params.z; % productivity


x_grid = policy.x_grid; % grid for real financial wealth
x_no = length(x_grid); % numer of grid points for financial wealth

SS_lambdaw = delta*SS_N/(1-SS_N); % implied job-finding probability
SS_theta = (SS_lambdaw/phi0)^(1/phi1); % labour market tigthness v/u

SS_J = psi*SS_theta/SS_lambdaw; % real price of equity

% transition probability for employment status next period, financial wealth, employment status this period
SS_prob = zeros(2,x_no,2); 
SS_prob(1,:,1) = 1-delta; 
SS_prob(2,:,1) = delta;
SS_prob(1,:,2) = SS_lambdaw;
SS_prob(2,:,2) = 1-SS_lambdaw;

SS_d = z - SS_w; % dividends
SS_tax = (mu*(1-SS_N)+B/SS_P/SS_w*(SS_R-1)/SS_R)/(SS_N+(1-SS_N)*mu); % tax rate
SS_income = repmat([1-SS_tax,(1-SS_tax)*mu]*SS_w,x_no,1); % after tax income of employed and unemployed


SS_x = repmat(reshape(x_grid,x_no,1),1,2); % replicate grid for employed and unemployed
SS_epsnext = repmat(reshape(1:2,[2,1,1]),[1,x_no,2]); % employment status next period

SS_c = policy.SS_c; % policy function for consumption
SS_b = policy.SS_b; % policy function for bonds
SS_q = (SS_x+SS_income-SS_c-SS_b/SS_R)/SS_J; % equity holdings from budget constraint


% Solve household's problem.
d2 = 1;
HHcrit = 1e-10;
HHdampening = 0.1;

smalleps = 1e-10; % small number to ensure that consumption, money holdings, ... is positive
tic

while d2 > HHcrit
    % guesses for policy functions
    tempc = SS_c;
    tempb = SS_b;
    tempq = SS_q;
    

    SS_xp(1,:,:) = (SS_d+SS_J*(1-delta)).*tempq+tempb/SS_P; % real financial wealth next period employed
    SS_xp(2,:,:) = (SS_d+SS_J*(1-delta)).*tempq+tempb/SS_P; % real financial wealth next period unemployed
    
    % interpolation with respect to cash on hand next period
    temp = interp1(x_grid,1:x_no,SS_xp,'linear','extrap');
    interpx_index = max(1,min(floor(temp),x_no-1));
    interpx_weight = 1+interpx_index-temp;    
    % consumption next period, depending on employment status next period, financial wealth, employment status this period
    SS_cnext = max(smalleps,interpx_weight.*SS_c(sub2ind([x_no,2],interpx_index,SS_epsnext))+(1-interpx_weight).*SS_c(sub2ind([x_no,2],interpx_index+1,SS_epsnext)));
    
    % expected marginal utility of consumption next period
    SS_MUcnext = squeeze(sum(SS_cnext.^(-gamma).*SS_prob,1));
    
    SS_c =(beta*(SS_d+SS_J*(1-delta))/SS_J*SS_MUcnext).^(-1/gamma); % consumption this period from FOC for equity
    if chi > 0
        SS_b = b_min+SS_R*SS_P*max(smalleps,(SS_c.^(-gamma)-beta*SS_R*SS_MUcnext)/chi).^(-1/zeta); % bond holdings this period from FOC for bonds
    else
        SS_b = b_min*ones(x_no,2); % bond holdings set at borrowing constraint (agents indifferent between both assets)
    end
    
    SS_q = (SS_x+SS_income-SS_c-SS_b/SS_R/SS_P)/SS_J; % equity holdings from budget constraint
    
    % Calculate policy functions for agents with binding borrowing constraint
    SS_q_binding = SS_q<q_min; % check for which agents borrowing constraint is binding
    SS_c_binding = min(SS_x+SS_income-SS_J*q_min-chi*smalleps,(beta*SS_R*SS_MUcnext+chi*((tempb-b_min)/SS_P/SS_R).^(-zeta)).^(-1/gamma)); % consumption from FOC for bonds
    SS_b_binding = (SS_x+SS_income-SS_c_binding-SS_J*q_min)*SS_R*SS_P; % bond holdings from budget constraint

    SS_c = SS_q_binding.*SS_c_binding + (1-SS_q_binding).*SS_c;
    SS_b = SS_q_binding.*SS_b_binding + (1-SS_q_binding).*SS_b;
    SS_q = SS_q_binding*q_min +         (1-SS_q_binding).*SS_q;
 
    %{
    % remove comment to draw deviation of policy function from guess
    plot(x_grid,[SS_q-tempq,SS_b-tempb,SS_c-tempc])
    drawnow
    %}
    
    % calculate deviations between guesses and new policy functions
    d1_c = max(max(abs(SS_c-tempc)./(1+tempc)));
    d1_m = max(max(abs(SS_b-tempb)./(1+tempb)));
    d1_q = max(max(abs(SS_q-tempq)./(1+tempq)));
    d2 = max([d1_c,d1_m,d1_q]);
    
    % update policy functions
    dampening = 100/(100+max(0,toc-5)^2)*HHdampening; % dampen more after longer running time
    SS_c = tempc+dampening*(SS_c-tempc);
    SS_b = tempb+dampening*(SS_b-tempb);
    SS_q = tempq+dampening*(SS_q-tempq);
    
if toc>60 % stop iterating, if it hasn't converged by that time
    disp(['Household problem: time out, error: ',num2str(d2)])
    plot(x_grid,[SS_q,SS_b,SS_c])
    drawnow;
    break
end

end

% check imposed minima and maxima in household problem
if min(SS_cnext(:)<= 2*smalleps)
    disp('HH problem, future consumption binding')
end

if chi>0
    test_b = (SS_c.^(-gamma)-beta*SS_R*SS_MUcnext)/chi;
    if min(test_b(:)<= 2*smalleps)
        disp('HH problem, bond holdings binding')
    end
    test_c = SS_x+SS_income-SS_J*q_min-chi*smalleps - SS_c_binding;
    if min(test_c(:)<= 2*smalleps)
        disp('HH problem, current consumption binding')
    end    
end

test_bc = abs(SS_x+SS_income-SS_c-SS_b/SS_R/SS_P-SS_J*SS_q);
if max(test_bc(:)>= smalleps)
    disp('HH problem, budget constraint not binding')
end

% calculate value function
SS_Utemp = (SS_c.^(1-gamma)-1)/(1-gamma)+chi*(((SS_b-b_min)/SS_R/SS_P).^(1-zeta)-1)/(1-zeta); % utility this period
SS_U = SS_Utemp/(1-beta); % initial guess
try
    SS_U = policy.SS_U; % load initial guess, if available
end

tic
d3=1;
while d3 > 1e-9
% calculate continuation value of utility
SS_Unext = interpx_weight.*SS_U(sub2ind([x_no,2],interpx_index,SS_epsnext))+(1-interpx_weight).*SS_U(sub2ind([x_no,2],interpx_index+1,SS_epsnext));
SS_Unew = SS_Utemp+beta*squeeze(sum(SS_Unext.*SS_prob,1));

d3 = max(abs((SS_Unew(:)-SS_U(:))./(1+SS_U(:))));
SS_U = SS_Unew;
if toc>10
disp('Value function iteration timed out')
break
end
end

% calculate implied bargaining power of worker = (U(emp)-U(unemp))/(U(emp)-U(unemp)+(1-tax)*U'(emp)*(J*(1-delta)+d))
tempworker = squeeze(SS_U(:,1)-SS_U(:,2)); % worker surplus

% find derivative of value function of worker with respect to financial wealth
tempderiv=zeros(x_no,1);
tempderiv(1) =  (SS_U(2,1)-SS_U(1,1))/(x_grid(2)-x_grid(1));
tempderiv(x_no) =  (SS_U(x_no,1)-SS_U(x_no-1,1))/(x_grid(x_no)-x_grid(x_no-1));
tempderiv(2:(x_no-1)) = 0.5*(SS_U(3:x_no,1)-SS_U(2:(x_no-1),1))'./(x_grid(3:x_no)-x_grid(2:(x_no-1)))+0.5*(SS_U(2:(x_no-1),1)-SS_U(1:(x_no-2),1))'./(x_grid(2:(x_no-1))-x_grid(1:(x_no-2)));

SS_bpower = tempworker./(tempworker+(1-SS_tax)*tempderiv*(SS_J*(1-delta)+SS_d));


%%%%%%%%% Aggregate to find new prices using histogram method
hist_no = 10*x_no; % use finer grid for financial wealth
hist_grid = linspace(x_grid(1),x_grid(x_no),hist_no)';

% Build transition matrix for financial wealth
hist_xp = zeros(2,hist_no,2);
for i=1:2 % employment this period
    for j=1:2 % employment next period
        hist_xp(j,:,i) = interp1(x_grid,squeeze(SS_xp(j,:,i)),hist_grid,'linear','extrap'); % financial wealth next period
    end
end
temp = interp1(hist_grid,1:hist_no,hist_xp,'linear','extrap');
hist_index = max(1,min(floor(temp),hist_no-1));
hist_weight = 1+hist_index-temp;
hist_matrix = cell(2,2); % transition matrix for financial wealth, for each combination of employment status this and next period
for i=1:2 % employment this period
    for j=1:2 % employment next period
        hist_matrix{i,j} = sparse(zeros(hist_no,hist_no)); % transition of financial wealth
        for k=1:hist_no
            hist_matrix{i,j}(k,hist_index(j,k,i)+(0:1)) = [hist_weight(j,k,i),1-hist_weight(j,k,i)];
        end
    end
end
% transition matrix including transition in labour market
hist_trans = [(1-delta)*hist_matrix{1,1},delta*hist_matrix{1,2};SS_lambdaw*hist_matrix{2,1},(1-SS_lambdaw)*hist_matrix{2,2}]; 

opts.disp = 0;

% Find Eigenvector corresponding to Eigenvalue 1
d_eig=1;
tic
while d_eig>0
    try
        [hist_V,~] = eigs(hist_trans'-eye(2*hist_no),1,0,opts);
        d_eig=0;
    catch err
        disp(err)
    end
    if toc>5
        disp('no Eigenvector found')
        hist_V=ones(hist_no,2);
        break
    end
end

hist_V = hist_V/sum(hist_V); % normalizing the distribution to mass 1
hist_dist(:,1) = hist_V(1:hist_no,1); % distribution of employed
hist_dist(:,2) = hist_V(hist_no+1:end,1); % distribution of unemployed

% interpolate policy functions on finer grid
for i=1:2
    hist_b(:,i) = interp1(x_grid,SS_b(:,i),hist_grid,'linear','extrap');
    hist_q(:,i) = interp1(x_grid,SS_q(:,i),hist_grid,'linear','extrap');
    hist_c(:,i) = interp1(x_grid,SS_c(:,i),hist_grid,'linear','extrap');
    hist_U(:,i) = interp1(x_grid,SS_U(:,i),hist_grid,'linear','extrap'); 
end
hist_bpower = interp1(x_grid,SS_bpower,hist_grid,'linear','extrap');
hist_assets = hist_q*SS_J+hist_b/SS_R/SS_P; % value of assets demanded

if chi==0 % bonds and equity perfect substitutes, w.l.o.g. each agent holds same proportion when market for equity clears
    SS_A = (SS_Q-q_min)*SS_J+(SS_B-b_min)/SS_R/SS_P; % aggregate demand for assets exceeding borrowing constraints
    tempshare = min(0,max(1,(SS_N-q_min)*SS_J/SS_A)); % share of equity in portfolio
    hist_b = b_min + (1-tempshare)*((hist_b-b_min)+(hist_q-q_min)*SS_J*SS_R*SS_P);
    hist_q = q_min + tempshare*((hist_b-b_min)/SS_R/SS_P/SS_J+(hist_q-q_min));
end

SS_Q = sum(sum(hist_dist.*hist_q)); % aggregate demand for shares
SS_B = sum(sum(hist_dist.*hist_b)); % aggregate demand for bonds   
    

% store policy functions
policy_new.x_grid = x_grid;
policy_new.SS_c = SS_c;
policy_new.SS_b = SS_b;
policy_new.SS_q = SS_q;
policy_new.SS_U = SS_U;
policy_new.dist = hist_dist;
policy_new.hist_grid = hist_grid;
policy_new.hist_c = hist_c;
policy_new.hist_b = hist_b;
policy_new.hist_q = hist_q;


% Compute median asset holdings
% find median agent
cumdist = cumsum(hist_dist);
sumdist = sum(hist_dist);
median_ind = max(1,min(hist_no-1,sum(cumdist<repmat(sumdist/2,hist_no,1)))); % index just below median agent

for i=1:2 % interpolate to get value for median agent
    stats.median.assets(i) = interp1(cumdist(median_ind(i)+(0:1),i),hist_assets(median_ind(i)+(0:1),i),sumdist(i)/2,'linear','extrap');
    stats.median.q(i) =      interp1(cumdist(median_ind(i)+(0:1),i),hist_q(median_ind(i)+(0:1),i),sumdist(i)/2,'linear','extrap');
    stats.median.b(i) =      interp1(cumdist(median_ind(i)+(0:1),i),hist_b(median_ind(i)+(0:1),i),sumdist(i)/2,'linear','extrap');
    stats.median.c(i) =      interp1(cumdist(median_ind(i)+(0:1),i),hist_c(median_ind(i)+(0:1),i),sumdist(i)/2,'linear','extrap');
end

% calculate averages
stats.avg.assets = sum(hist_dist.*hist_assets)./sum(hist_dist);
stats.avg.q = sum(hist_dist.*hist_q)./sum(hist_dist);
stats.avg.b = sum(hist_dist.*hist_b)./sum(hist_dist);
stats.avg.c = sum(hist_dist.*hist_c)./sum(hist_dist);
stats.avg.value =  sum(hist_dist.*hist_U)./sum(hist_dist);
stats.avg.bpower =  sum(hist_dist(:,1).*hist_bpower)/sum(hist_dist(:,1));

stats.SS_J = SS_J; % price of equity

dev = [SS_Q-SS_N,SS_B-B]; % deviation from market clearing for equity and bond market
