function [sim_results,Np,P,d1,value,IRF_results] = repagent_simulation(params,sim_p,Np,P)

w_coeff = params.w_coeff;
phi0 = params.phi0;
phi1 = params.phi1;
beta = params.beta;
gamma = params.gamma;
delta = params.delta;
psi = params.psi;
B = params.B;
mu = params.mu; 
z = params.z;
z_prob = params.z_prob;
N = params.N;

z_no=length(z);
grid_Nno = length(N);

if length(mu) == 1 % if mu is scalar, use same mu in each aggregate state
    mu = mu*ones(z_no,1); 
end

if nargin<3
    Np = repmat(N,z_no,1);% initial guess
end
if nargin<4
    P = ones(z_no,grid_Nno); % initial guess
end

% find policy functions
[Np,J,P,d1,meanP,value,Nlimit,R] = repagent_policy(params,N,Np,P);



sim_no = length(sim_p);
sim_z = z(sim_p)'; % productivity


% preallocate simulated variables
sim_N=zeros(sim_no,1);
sim_J=zeros(sim_no,1);
sim_P=zeros(sim_no,1);
sim_Jp=zeros(sim_no,1);
sim_Pp=zeros(sim_no,1);
sim_w=zeros(sim_no,1);
sim_tax=zeros(sim_no,1);
sim_Pbp=zeros(sim_no,1);
sim_dp=zeros(sim_no,1);
sim_B=zeros(sim_no,1);
sim_Q=zeros(sim_no,1);
sim_Qp=zeros(sim_no,1);
sim_C=zeros(sim_no,1);
sim_Erq=zeros(sim_no,1);
sim_Einf=zeros(sim_no,1);
sim_Erb=zeros(sim_no,1);
sim_MRS=zeros(sim_no,1);
sim_theta=zeros(sim_no,1);
sim_value=zeros(sim_no,1);

lambdaf = psi/(beta*(1-w_coeff(1))/(1-beta*(1-delta))); % in steady state
lambdaw = (phi0/lambdaf^phi1)^(1/(1-phi1));
sim_N(1) = lambdaw/(delta+lambdaw); % start with steady state employment


for t=1:sim_no
    if t>1
        sim_N(t) = sim_Q(t-1); % employment as determined in last period
    end
    
    % interpolate policy functions to find J and P
    sim_J(t) = interp1(N,J(sim_p(t),:),sim_N(t),'linear','extrap');
    sim_P(t) = interp1(N,P(sim_p(t),:),sim_N(t),'linear','extrap');
    
    % perceived and true prices are the same for rep agent
    sim_Jp(t) = sim_J(t); 
    sim_Pp(t) = sim_P(t);

    sim_w(t) = w_coeff(1)*sim_z(t)^w_coeff(2)*sim_Pp(t)^(w_coeff(3)-1)*meanP^(1-w_coeff(3)); % real wage

    % Price of bond
    sim_Pbp(t) = 1/interp1(N,R(sim_p(t),:),sim_N(t),'linear','extrap');
    
    sim_tax(t) =  1-(sim_N(t)*sim_w(t)-(1-sim_Pbp(t))*B/sim_Pp(t))/((1-sim_N(t))*mu(sim_p(t))+sim_N(t))/sim_w(t); % tax
    sim_dp(t) = sim_z(t)-sim_w(t); % perceived real dividend

    
    sim_B(t) = B/sim_P(t); % demand for real bonds
    
    sim_theta(t) = (sim_J(t)/psi*phi0)^(1/(1-phi1)); % market tightness
    sim_templambdaw = phi0*sim_theta(t)^phi1; % transition probability for unemployed
    
    sim_Q(t) = (1-delta)*sim_N(t)+sim_templambdaw*(1-sim_N(t)); % demand for equity
    sim_Qp(t) = sim_Q(t); % perceived and true demand is the same for rep agent
    
    sim_C(t) = sim_z(t).*sim_N(t)-psi*sim_theta(t).*(1-sim_N(t)); % consumption
    
    % calculate prices next period for each possible state to calculate
    % expected return on equity, and the MRS
    for i=1:z_no
        tempJp(i) =  interp1(N,J(i,:),sim_Q(t),'linear','extrap');
        tempPp(i) =  interp1(N,P(i,:),sim_Q(t),'linear','extrap');
    end
    tempwp = w_coeff(1)*z.^w_coeff(2).*tempPp.^(w_coeff(3)-1);
    tempdp = z-tempwp;
    tempCp = z*sim_Q(t)-psi*(tempJp/psi*phi0).^(1/(1-phi1))*(1-sim_Q(t));
    tempprob = z_prob(sim_p(t),:);
    
    sim_Erq(t) = (tempJp*(1-delta)+tempdp)/sim_J(t)*tempprob' -1; % expected return on equity
    sim_Einf(t) = (tempPp/sim_P(t))*tempprob'-1; % expected inflation rate
    sim_Erb(t) = ((sim_P(t)./tempPp)*tempprob')/sim_Pbp(t)-1; % expected return on bonds

    sim_MRS(t) = (tempCp/sim_C(t)).^(-gamma)*tempprob';
    
    
    sim_value(t) = interp1(N,value(sim_p(t),:),sim_N(t),'linear','extrap');
    
end

sim_results=[sim_z,sim_N,sim_J,sim_Jp,sim_P,sim_Pp,sim_C,sim_w,sim_tax,sim_Pbp,sim_dp,sim_theta,sim_Erb,sim_Erq,sim_Einf,sim_MRS,sim_value];


if min(sim_results(:,2))<N(1)
    disp('simulation: downward extrapolation necessary')
end

if max(sim_results(:,2))>N(end)
    disp('simulation: upward extrapolation necessary')
end


% Calculate impulse-response functions
if z_no==2
    % variables on finer grid, used for calculation of IRFs
    IRF_gridno = 500;
    IRF_N = repmat(linspace(N(1),N(grid_Nno),IRF_gridno)',1,z_no);
    IRF_z = repmat(z,IRF_gridno,1); %productivity
     
    for i=1:z_no
        IRF_Np(:,i) = interp1(N,Np(i,:),IRF_N(:,1),'linear','extrap'); % employment next period
        IRF_J(:,i) = interp1(N,J(i,:),IRF_N(:,1),'linear','extrap'); % real price of equity
        IRF_P(:,i) = interp1(N,P(i,:),IRF_N(:,1),'linear','extrap'); % price level
        for j=1:z_no % productivity next period
           temp_Pp(:,j) =  interp1(N,P(j,:),IRF_Np(:,i),'linear','extrap'); % % price level next period
        end
        IRF_EPpP(:,i) = (temp_Pp*z_prob(i,:)')./IRF_P(:,i) ; % expected P'/P
        IRF_R(:,i) = interp1(N,R(i,:),IRF_N(:,1),'linear','extrap');
    end
    IRF_w = w_coeff(1)*IRF_z.^w_coeff(2).*IRF_P.^(w_coeff(3)-1)*meanP^(1-w_coeff(3)); % real wage

    IRF_tax = 1-(IRF_N.*IRF_w-(IRF_R./(1+IRF_R))*B./IRF_P)./((1-IRF_N).*repmat(mu',IRF_gridno,1)+IRF_N)./IRF_w; % tax rate
    IRF_taxvol = IRF_tax.*IRF_w.*(IRF_N+repmat(mu',IRF_gridno,1).*(1-IRF_N)); % tax volume
    IRF_Rreal = IRF_R./IRF_EPpP-1; % expected real interest rate
    IRF_SIR = 0*IRF_z;
    IRF_SIRreal = 0*IRF_z;
    IRF_C = IRF_z.*IRF_N-psi*(1-IRF_N).*(IRF_J/psi*phi0).^(1/(1-phi1)); % consumption
    
    IRF_options.IRF_no = 51; % number of periods for IRF
    IRF_options.zstart = [2,1];  % start with good state, then drop to bad state
    IRF_options.percentiles = [0.05,0.95,0.5,0.1,0.9];
    IRF_options.distinit = zeros(IRF_gridno*z_no,1);
    % start at limiting value of employment
    temp = interp1(IRF_N(:,1),1:IRF_gridno,Nlimit(IRF_options.zstart(1)),'linear');
    tempindex = floor(temp);
    tempweight = 1+tempindex-temp;
    IRF_options.distinit((IRF_options.zstart(1)-1)*IRF_gridno+tempindex,1) = tempweight; 
    IRF_options.distinit((IRF_options.zstart(1)-1)*IRF_gridno+tempindex+1,1) = 1-tempweight; 
    IRF_vars = {IRF_z,IRF_N,IRF_z.*IRF_N,IRF_w,IRF_J,IRF_P,IRF_R,IRF_tax,IRF_taxvol,IRF_Rreal,IRF_w./IRF_z,IRF_SIR,IRF_SIRreal,IRF_C};
    IRF_devs = {'log','ppt','log','log','log','log','ppt','ppt','log','ppt','log','ppt','ppt','log'}; % deviation of variables to be calculated (log or ppt)

    % Compute IRF
    IRF = IRF_calculation(IRF_N(:,1),IRF_Np,z_prob,IRF_options,IRF_vars);
    % Compute path without shock
    IRF_options.zstart = [2];
    IRF0 = IRF_calculation(IRF_N(:,1),IRF_Np,z_prob,IRF_options,IRF_vars);

    for k=1:length(IRF_vars) % calculate deviations
        switch IRF_devs{k}
            case 'log'
                IRF_results(:,k) = log(IRF.mean{k}./IRF0.mean{k});
            case 'ppt'
                IRF_results(:,k) = (IRF.mean{k}-IRF0.mean{k})*100;
        end
    end

end
    
    

function [Np,J,P,d1,meanP,value,Nlimit,R] = repagent_policy(params,grid_N,Np,P)
% repagent: on a grid for N find policy functions N' and P
% given a guess, calculate conditional C' and right hand side of equity
% Euler equation --> updated C
% then use bond FOC to get updated P
w_coeff = params.w_coeff;
phi0 = params.phi0;
phi1 = params.phi1;
beta = params.beta;
gamma = params.gamma;
zeta = params.zeta;
delta = params.delta;
psi = params.psi;
chi = params.chi;
B = params.B;
b_min = params.b_min;
z = params.z;
z_prob = params.z_prob;
IR_coeff = params.IR_coeff;

z_no = length(z);
grid_Nno = length(grid_N);

zL = repmat(z',1,grid_Nno); % replicate z for each possible N
NL = repmat(grid_N,z_no,1); % replicate grid for N for each possible z
z_probL = repmat(z_prob',[1,1,grid_Nno]); % replicate transition probabilities for each possible N

d1=1;

dampeningN = 0.9;
dampeningP = 0.9;
tic

while d1>1e-8
theta = (max(1e-10,(Np-(1-delta)*NL))./(1-NL)/phi0).^(1/phi1); % market tightness implied by Np and matching function
J = psi/phi0*theta.^(1-phi1); % J from zero profit condition
C = zL.*NL-psi*theta.*(1-NL); % C from budget constraint

% find limiting values for N and P for each z
options = optimset('Display','off','tolFun',1e-12,'tolX',1e-12);
for i=1:z_no
   Nlimit(i)=fsolve(@(x)(interp1(grid_N,Np(i,:)-grid_N,x,'linear')),mean(grid_N),options); % long run value 
   Ptemp(i)=interp1(grid_N,P(i,:),Nlimit(i),'linear','extrap');
end

[tempv,~]=eigs(z_prob',1);
tempv=tempv/sum(tempv);

meanP = Ptemp*tempv; % P in wage equation is normalized by average P at the limiting values
w = w_coeff(1)*zL.^w_coeff(2).*P.^(w_coeff(3)-1)*meanP^(1-w_coeff(3)); % real wage
d = zL-w; % dividends

for i=1:z_no % productivity next period
    Cp(i,:,:) = interp1(grid_N,C(i,:),Np,'linear','extrap'); % consumption next period
    Jdp(i,:,:) = interp1(grid_N,J(i,:)*(1-delta)+d(i,:),Np,'linear','extrap'); % (1-delta)*J+d next period
    Pp(i,:,:) = interp1(grid_N,P(i,:),Np,'linear','extrap'); % P next period
end

% Interest rate rule
R = max(IR_coeff(3),IR_coeff(1)*(P/meanP).^IR_coeff(2)); % gross nominal interest rate (1+i)=IR_coeff(1)*(P/meanP)^IR_coeff(2)



EMUq = max(1e-10,beta*squeeze(sum(Cp.^(-gamma).*Jdp.*z_probL,1))); % beta*E[C'^-gamma*((1-delta)*J'+d'))]
EMUm = beta*R.*squeeze(sum(Cp.^(-gamma)./Pp.*z_probL,1));  % beta*R*E[C'^-gamma*P']

C_new = (EMUq./J).^(-1/gamma); % C from FOC for equity
P_new = max(0.1,(C_new.^(-gamma)-chi*((B-b_min)./P).^(-zeta))./EMUm); % P from FOC for bonds

theta_new = max(1e-3,(zL.*NL-C_new))/psi./(1-NL); % theta from budget constraint
Np_new = (1-delta)*NL+phi0*theta_new.^phi1.*(1-NL); % N' from matching function

d1_P = max(max(abs(P_new-P)./P));
d1_N = max(max(abs(Np_new-Np)./Np));
d1 = max(d1_P,d1_N);

P = dampeningP*P + (1-dampeningP)*P_new;
Np = dampeningN*Np + (1-dampeningN)*Np_new;

if toc>120
    dampeningN = 0.95;
    dampeningP = 0.95;
end
if toc>400
    dampeningN = 0.999;
    dampeningP = 0.999;
end

if toc>600
    % break if it takes too long
    disp('rep agent problem not fully solved')
    d1
    break
end
end

% Calculate value function
d2=1;
value = 0*C;
while d2>1e-8

for i=1:z_no % productivity next period
    valuep(i,:,:) = interp1(grid_N,value(i,:),Np,'linear','extrap'); % value function next period
end

value_new = (C.^(1-gamma)-1)/(1-gamma)+chi*((B./P).^(1-zeta)-1)/(1-zeta)+beta*squeeze(sum(valuep.*z_probL,1));

d2 = max(max(abs(value_new-value)./abs(1+value)));

value = value_new;
end

