clear all
close all

% load file with calibrated baseline and coefficients for aggregate laws of
% motion for various alternatives
load revised2_coeffs

trendinflation = (1.0087)^0.25-1; % quarterly trend inflation

%%%%% Solve baseline
disp('#####################################################')
disp('Solve baseline')
disp('#####################################################')

params = baseline.params; % structure containing parameters
coeffs = baseline.coeffs; % coefficients of aggregate laws

rng default
policy_name = [savename,'_policy'];

% simulate producitivity shock
sim_no = 2000; %number of periods in simulation
sim_p = zeros(sim_no,1);
sim_p(1) = round(length(params.z)/2); % productivity state
for t=2:sim_no
    sim_p(t) = 1+sum(rand>cumsum(params.z_prob(sim_p(t-1),:)));
end

% find number of dropped periods between 150-300 such that 
% average log-productivity in remaining periods is close to 0
tempmean = NaN(sim_no,1);
for t=150:300
    tempmean(t) = mean(log(params.z(sim_p((t+1):end))));
end
[~,sim_drop]=min(abs(tempmean));

% define options for solution and simulation
options.policy_name=policy_name; % savename for policy functions
options.HHcrit = 1e-8; % accuracy of household problem
options.timefactor = 1; % time allowed for convergence of household problem
options.sim_individual_no=1000; % number of agents when simulating them separately
options.sim_p = sim_p; % series of aggregate productivity states used for simulation
options.sim_no = sim_no; % number of periods simulated
options.sim_drop = sim_drop; % number of initial periods dropped
options.aggcritvalue = 1e-6; % precision needed for aggregate laws of motion
options.plotpoints = 500; % subsample of points drawn for each curve when plotting cross-sectional observations
targets.N=0.893; % targeted mean employment


% save initial guess for policy functions
grid_N=baseline.policy.grid_N; % grid for aggregate employment
x_grid=baseline.policy.x_grid; % grid for real financial wealth
aggb=baseline.policy.aggb; % demand for bonds
aggq=baseline.policy.aggq; % demand for equity
aggc=baseline.policy.aggc; % demand for consumption
save(options.policy_name,'grid_N','x_grid','aggb','aggq','aggc')


if norm(het_aggrisk(params,coeffs,options))>1e-6 % if baseline is not accurate, solve for coefficients
    solver_options = optimset('Display','iter','TolFun',1e-6,'TolX',1e-6,'UseParallel',false,'MaxFunEvals',500);
    
    % when using the solver, put more weight on linear and quadratic coefficients
    % as absolute changes have smaller effects on the solution
    weight=ones(12,1); 
    weight([3,4,7,8]) = (grid_N(end)-grid_N(1))/2;
    weight(9:12) = ((grid_N(end)-grid_N(1))/2)^2;
    law_weight=1./weight'*100;
    
    % solve for laws of motion without recalibrating
    xsolved = fsolve(@(x)solve_calib(targets,law_weight,params,x,options),coeffs./law_weight,solver_options);
    coeffs = law_weight.*xsolved;
end

baseline.params = params; % store baseline parameters
baseline.coeffs = coeffs; % store baseline coefficients

[dev,sim_results,sim_details] = het_aggrisk(baseline.params,baseline.coeffs,options);
if norm(dev)>1e-6
   disp(['###### Warning: aggregate laws not fully converged, dev=',num2str(dev)])
end
baseline.IRF=sim_details.IRF; % store baseline IRF
baseline.sim_results = sim_results; % store baseline time series
baseline.dev = dev; % store baseline deviation
clear sim_details

baseline.policy = load(options.policy_name,'grid_N','x_grid','aggb','aggq','aggc'); % store baseline policy functions


%%%%% Solve baseline without aggregate risk
disp('#####################################################')
disp('Solve baseline without aggregate risk')
disp('#####################################################')

% parameters same as baseline except for productivity z=1 always
SS_params = baseline.params; 
SS_params.z = 1;
SS_params.z_prob = 1;

try 
    % if steady state structure exists, use initial guesses for employment and price level
    SS_params.SS_N = baseline.SS.params.SS_N;
    SS_params.SS_P = baseline.SS.params.SS_P;
catch
    SS_params.SS_N = targets.N;
    SS_params.SS_P = 1;
end

[baseline.SS,SS_policy] = solve_steadystate(SS_params); % solve steady state
baseline.SS.policy = SS_policy; % story policy function in addition


disp('#####################################################')
disp('Solve baseline with representative agent')
disp('#####################################################')
% solve representative agent economy with average employment being the same
% as in baseline
try
    % use existing guess for beta
    baseline = repagent(baseline,baseline.rep.params.beta); 
catch
    % use beta=0.99
    baseline = repagent(baseline,0.99); 
end
% check whether mean employment is sufficiently close to heterogeneous
% agent economy
if norm(mean(baseline.rep.sim_results((options.sim_drop+1):end,2))-mean(baseline.sim_results((options.sim_drop+1):end,2)))>1e-6
    % calibrate beta
    solver_options = optimset('Display','iter','TolFun',1e-6,'TolX',1e-6,'UseParallel',false,'MaxFunEvals',500);
    xsolved = fsolve(@(x)repagent_calibration(baseline,100*x,options.sim_drop),baseline.rep.params.beta/100,solver_options);
    baseline = repagent(baseline,100*xsolved); 
end
clear sim_details
save(savename)



%% Solve for different values of PLT
disp('#####################################################')
disp('Solve for different parameter values of price-level-targeting')
disp('#####################################################')

PLT_grid = [0:0.025:0.1,0.15:0.05:0.3]; % values for nu_P considered

for PLT_iter=1:length(PLT_grid)
    % set all parameters but nu_P equal to baseline
    params = baseline.params;
    coeffs = baseline.coeffs;
    PLT_target.PLT=PLT_grid(PLT_iter);
    disp(['PLT=',num2str(PLT_target.PLT)])    

    try % use existing solution, if available
        coeffs = store_PLT{PLT_iter}.coeffs;
        params.IR_coeff(2) = PLT_target.PLT;
    catch % use closest existing solution
        temp = sortrows([1:length(PLT_grid);-abs(PLT_grid-PLT_target.PLT)]',2);
        for i = 1:size(temp,1)
           try
                coeffs = store_PLT{temp(i,1)}.coeffs;
                params.IR_coeff(2) = store_PLT{temp(i,1)}.params.IR_coeff(2);
           end

        end

    end
    
    grid_N=baseline.policy.grid_N;
    try  % use existing policy functions
        x_grid=store_PLT{PLT_iter}.policy.x_grid;
        aggb=store_PLT{PLT_iter}.policy.aggb;
        aggq=store_PLT{PLT_iter}.policy.aggq;
        aggc=store_PLT{PLT_iter}.policy.aggc;        
    catch % use baseline policy functions
        x_grid=baseline.policy.x_grid;
        aggb=baseline.policy.aggb;
        aggq=baseline.policy.aggq;
        aggc=baseline.policy.aggc;   
    end
    save(options.policy_name,'grid_N','x_grid','aggb','aggq','aggc')
    
    dev = solve_PLT(PLT_target,ones(1,12),params,[coeffs,params.IR_coeff(2)],options);
    if norm(dev)>options.aggcritvalue % if deviation too large use solver
        % when using the solver, put more weight on linear and quadratic coefficients
        % as absolute changes have smaller effects on the solution        
        weight=ones(12,1);
        weight([3,4,7,8]) = (grid_N(end)-grid_N(1))/2;
        weight(9:12) = ((grid_N(end)-grid_N(1))/2)^2;
        law_weight=1./weight'*100;        
        solver_options = optimset('Display','iter','TolFun',1e-6,'TolX',1e-6,'UseParallel',false,'MaxFunEvals',500);
        % solve for coefficients in aggregate laws and interest rate rule
        % coefficient
        xsolved = fsolve(@(x)solve_PLT(PLT_target,law_weight,params,x,options),[coeffs./law_weight,params.IR_coeff(2)],solver_options);
        coeffs = law_weight.*xsolved(1:12);
        params.IR_coeff(2) = xsolved(13);
        dev = solve_PLT(PLT_target,law_weight,params,[coeffs./law_weight,params.IR_coeff(2)],options);
    end
    if norm(dev)>options.aggcritvalue
        disp(['###### Warning: aggregate laws or PLT-coefficient not fully converged, dev=',num2str(dev)])
    end
    
    [~,sim_results,sim_details] = het_aggrisk(params,coeffs,options); % simulate economy in detail
    
    store_PLT{PLT_iter}.params = params; % store parameters
    store_PLT{PLT_iter}.coeffs = coeffs; % store coefficients of aggregate laws
    store_PLT{PLT_iter}.dev = dev; % store deviation of aggregate laws and PLT coefficient
    store_PLT{PLT_iter}.sim_results = sim_results; % store simulated time series
    store_PLT{PLT_iter}.IRF = sim_details.IRF; % store IRFs
    store_PLT{PLT_iter}.policy = load(options.policy_name,'grid_N','x_grid','aggb','aggq','aggc'); % store policy functions
end
clear sim_details
save(savename)

%% Calculate path with monetary policy at ZLB
disp('#####################################################')
disp('Solve for path in recession with nominal interest rate at ZLB')
disp('#####################################################')

params = baseline.params;
coeffs = baseline.coeffs;

[~,sim_results,sim_details] = het_aggrisk(params,coeffs,options); % get baseline distribution from end of boom

init_no = 10;
init.altz = params.z(1)*ones(init_no,1); % productivity in alternative third state
init.state_expected = repmat([0, params.z_prob(1,2),params.z_prob(1,1)],init_no-1,1); % probability of continuing in one of the 3 states (last period has to have 0 probability to remain in 3rd regime)
init.state_expected(init_no,:)=[params.z_prob(1,1:2),0]; % usual continuation probabilities in last period
init.dist = sim_details.store_dist{2}; % take distribution from end of boom
init.sim_b = sim_details.store_sim_b{2}; % take asset holdings from end of boom
init.sim_q = sim_details.store_sim_q{2}; % take equity holdings from end of boom

load(options.policy_name,'grid_N','x_grid','aggb','aggq','aggc'); 

for t=1:init_no % initial guess for policy functions
    altb{t} = squeeze(aggb(1,1,:,:));
    altq{t} = squeeze(aggq(1,1,:,:));
    altc{t} = squeeze(aggc(1,1,:,:));
end

altoptions = options;
altoptions.timefactor = 0.3;

save(altoptions.policy_name,'grid_N','x_grid','aggb','aggq','aggc','altb','altq','altc')

try % use existing solution
    altJ=store_alt.altJ;
    altP=store_alt.altP;
catch % initial guess for J and P on alternative path
    altJ = sim_results(sim_details.t_longrun(1),3)*ones(init_no,1);
    altP = sim_results(sim_details.t_longrun(1),5)*ones(init_no,1);
end

altPb = (1+trendinflation)*ones(init_no,1); % bond price on alternative path at ZLB

% solve for alternative path
dev = alternativepath(params,coeffs,altoptions,init,altJ,altP,altPb);
if norm(dev)>options.aggcritvalue % if deviation too large use solver
    solver_options = optimset('Display','iter','TolFun',1e-6,'TolX',1e-6,'UseParallel',false,'MaxFunEvals',500);
    weight = 10;
    xsolved = fsolve(@(x)alternativepath(params,coeffs,altoptions,init,weight*x(:,1),weight*x(:,2),altPb),[altJ,altP]/weight,solver_options);
    altJ=weight*xsolved(:,1);
    altP=weight*xsolved(:,2);
end
[dev,alt_sim_results1,alt_sim_final] = alternativepath(params,coeffs,altoptions,init,altJ,altP,altPb);
if norm(dev)>options.aggcritvalue
    disp(['###### Warning: aggregate laws not fully converged, dev=',num2str(dev)])
end

store_alt.altJ = altJ; % store J on alternative path
store_alt.altP = altP; % store P on alternative path
store_alt.dev = dev; % store deviation of aggregate laws
store_alt.sim_results = alt_sim_results1; % store simulated time series

% continue recession with baseline law of motion
options_recession = options;
options_recession.init_dist = alt_sim_final.dist; % distribution from end of alternative path
options_recession.init_q = alt_sim_final.q; % equity holdings from end of alternative path
options_recession.init_b = alt_sim_final.b; % equity holdings from end of alternative path
options_recession.sim_no = 10; % additional periods in recession
options_recession.sim_drop = 0; % no initial periods are dropped
options_recession.sim_p = ones(options_recession.sim_no,1); % stay in recession

[~,alt_sim_results2] = het_aggrisk(params,coeffs,options_recession);

% combine simulated results from last period of boom, alternative path, and
% continuation in recession
alt_sim_results=[sim_results(sim_details.t_longrun(2),:);alt_sim_results1;alt_sim_results2];

% Calculate IRFs as in sustainedIRF.m
% productivity, output, employment, unit labor costs, real wage, asset
% prices, price level, unemployment rate, real interest rate, nominal
% interest rate
IRF_vars = {alt_sim_results(:,1),alt_sim_results(:,1).*alt_sim_results(:,2),alt_sim_results(:,2),alt_sim_results(:,8)./alt_sim_results(:,1),alt_sim_results(:,8),alt_sim_results(:,3),alt_sim_results(:,5),1-alt_sim_results(:,2),alt_sim_results(:,13),(1+trendinflation)./alt_sim_results(:,10)-1};
IRF_devs = {'log','log','ppt','log','log','log','log','ppt','ppt','ppt'}; % deviation of variables to be calculated (log or ppt)
clear IRF
for k=1:length(IRF_vars) % calculate deviations
    switch IRF_devs{k}
        case 'log'
            IRF(:,k) = log(IRF_vars{k}/IRF_vars{k}(1));
        case 'ppt'
            IRF(:,k) = (IRF_vars{k}-IRF_vars{k}(1))*100;
    end
end
IRF(:,11) = ((1+trendinflation)./alt_sim_results(:,10)).^4-1; % annualized nominal interest rate after adding trend inflation
store_alt.IRF=IRF;

clear sim_details
save(savename)


%% Solve for different values of nominal wage rigidity
disp('#####################################################')
disp('Solve for different values of nominal wage rigidity')
disp('#####################################################')

wP_grid = [0.7:0.05:1,0.65:(-0.05):0.5]; % values for omega_P considered


for wP_iter=1:length(wP_grid)
    % set all parameters but omega_P equal to baseline
    params = baseline.params;
    coeffs = baseline.coeffs;
    wP_target.wP=wP_grid(wP_iter);
    disp(['wP=',num2str(wP_target.wP)])
    
    try % use existing solution
        coeffs = store_wP{wP_iter}.coeffs;
        params.w_coeff(3) = wP_target.wP;
    catch % use closest existing solution
        temp = sortrows([1:length(wP_grid);-abs(wP_grid-wP_target.wP)]',2);
        for i = 1:size(temp,1)
           try
                coeffs = store_wP{temp(i,1)}.coeffs;
                params.w_coeff(3) = store_wP{temp(i,1)}.params.w_coeff(3);
           end

        end
    end
    
    grid_N=baseline.policy.grid_N;
    try  % use existing policy functions
        x_grid=store_wP{wP_iter}.policy.x_grid;
        aggb=store_wP{wP_iter}.policy.aggb;
        aggq=store_wP{wP_iter}.policy.aggq;
        aggc=store_wP{wP_iter}.policy.aggc;        
    catch % use baseline policy functions
        x_grid=baseline.policy.x_grid;
        aggb=baseline.policy.aggb;
        aggq=baseline.policy.aggq;
        aggc=baseline.policy.aggc;   
    end
    save(options.policy_name,'grid_N','x_grid','aggb','aggq','aggc')
    
    dev = solve_wP(wP_target,ones(1,12),params,[coeffs,params.w_coeff(3)],options);
    if norm(dev)>options.aggcritvalue % if deviation too large use solver
        % when using the solver, put more weight on linear and quadratic coefficients
        % as absolute changes have smaller effects on the solution        
        weight=ones(12,1);
        weight([3,4,7,8]) = (grid_N(end)-grid_N(1))/2;
        weight(9:12) = ((grid_N(end)-grid_N(1))/2)^2;
        law_weight=1./weight'*100;        
        solver_options = optimset('Display','iter','TolFun',1e-6,'TolX',1e-6,'UseParallel',false,'MaxFunEvals',500);
        % solve for coefficients in aggregate laws and wage coefficient
        xsolved = fsolve(@(x)solve_wP(wP_target,law_weight,params,x,options),[coeffs./law_weight,params.w_coeff(3)],solver_options);
        coeffs = law_weight.*xsolved(1:12);
        params.w_coeff(3) = xsolved(13);
        dev = solve_wP(wP_target,law_weight,params,[coeffs./law_weight,params.w_coeff(3)],options);
    end
    if norm(dev)>options.aggcritvalue
        disp(['###### Warning: aggregate laws or wage-coefficient not fully converged, dev=',num2str(dev)])
    end
    
    [~,sim_results,sim_details] = het_aggrisk(params,coeffs,options); % simulate economy in detail
    store_wP{wP_iter}.params = params; % store parameters
    store_wP{wP_iter}.coeffs = coeffs; % store coefficients of aggregate laws
    store_wP{wP_iter}.dev = dev; % store deviation of aggregate laws and PLT coefficient
    store_wP{wP_iter}.sim_results = sim_results; % store simulated time series
    store_wP{wP_iter}.IRF = sim_details.IRF; % store IRFs
    store_wP{wP_iter}.policy = load(options.policy_name,'grid_N','x_grid','aggb','aggq','aggc'); % store policy functions
    
    if wP_target.wP==baseline.params.w_coeff(3) || wP_target.wP == 1 % in case of baseline and with no nominal wage rigidity add rep agent solution
        store_wP{wP_iter} = repagent(store_wP{wP_iter},baseline.rep.params.beta); % add rep agent solution with beta from baseline rep agent solution
    end    
end
clear sim_details
save(savename)


%% Solve for different values of unemployment insurance without PLT
disp('#####################################################')
disp('Solve for different values of unemployment insurance without PLT')
disp('#####################################################')

mu_grid_PLT = [0.5:0.01:0.7,0.49:(-0.01):0.3]; % values for mu considered

for mu_iter=1:length(mu_grid_PLT)
    % use solution with constant nominal interest rate
    params = store_PLT{1}.params;
    coeffs = store_PLT{1}.coeffs;
    
    % set targeted value of unemployment insurance
    mu_target.mu=mu_grid_PLT(mu_iter);
    disp(['mu=',num2str(mu_target.mu)])

    try % use existing solution
        coeffs = store_mu_PLT{mu_iter}.coeffs;
        params.mu = mu_target.mu;
    catch % use closest existing solution
        temp = sortrows([1:length(mu_grid_PLT);-abs(mu_grid_PLT-mu_target.mu)]',2);
        for i = 1:size(temp,1)
           try
                coeffs = store_mu_PLT{temp(i,1)}.coeffs;
                params.mu = store_mu_PLT{temp(i,1)}.params.mu;
           end

        end
    end
    
    grid_N=baseline.policy.grid_N;
    try  % use existing policy functions
        x_grid=store_mu_PLT{mu_iter}.policy.x_grid;
        aggb=store_mu_PLT{mu_iter}.policy.aggb;
        aggq=store_mu_PLT{mu_iter}.policy.aggq;
        aggc=store_mu_PLT{mu_iter}.policy.aggc;        
    catch % use policy functions with baseline mu
        x_grid=store_PLT{1}.policy.x_grid;
        aggb=store_PLT{1}.policy.aggb;
        aggq=store_PLT{1}.policy.aggq;
        aggc=store_PLT{1}.policy.aggc;   
    end
    save(options.policy_name,'grid_N','x_grid','aggb','aggq','aggc')  
    
    dev = solve_mu(mu_target,ones(1,length(coeffs)),params,[coeffs,params.mu],options);
    if norm(dev)>options.aggcritvalue % if deviation too large use solver
        % when using the solver, put more weight on linear and quadratic coefficients
        % as absolute changes have smaller effects on the solution   
        weight=ones(12,1);
        weight([3,4,7,8]) = (grid_N(end)-grid_N(1))/2;
        weight(9:12) = ((grid_N(end)-grid_N(1))/2)^2;
        law_weight=1./weight'*100;        
        solver_options = optimset('Display','iter','TolFun',1e-6,'TolX',1e-6,'UseParallel',false,'MaxFunEvals',500);
        % solve for coefficients in aggregate laws and unemployment
        % insurance
        xsolved = fsolve(@(x)solve_mu(mu_target,law_weight,params,x,options),[coeffs./law_weight,params.mu],solver_options);
        coeffs = law_weight.*xsolved(1:12);
        params.mu = xsolved(13);
        dev = solve_mu(mu_target,ones(1,length(coeffs)),params,[coeffs,params.mu],options);
    end
    if norm(dev)>options.aggcritvalue
        disp(['###### Warning: aggregate laws or unemployment insurance coefficient not fully converged, dev=',num2str(dev)])
    end
    
    [~,sim_results,sim_details] = het_aggrisk(params,coeffs,options); % simulate economy in detail
    store_mu_PLT{mu_iter}.params = params; % store parameters
    store_mu_PLT{mu_iter}.coeffs = coeffs; % store coefficients of aggregate laws
    store_mu_PLT{mu_iter}.dev = dev; % store deviation of aggregate laws and PLT coefficient
    store_mu_PLT{mu_iter}.sim_results = sim_results; % store simulated time series
    store_mu_PLT{mu_iter}.IRF = sim_details.IRF; % store IRFs
    store_mu_PLT{mu_iter}.policy = load(options.policy_name,'grid_N','x_grid','aggb','aggq','aggc'); % store policy functions
    
    % calculate solution without aggregate risk
    disp(['Steady State: mu=',num2str(mu_grid_PLT(mu_iter))])
    SS_params = baseline.SS.params;
    SS_params.mu=mu_grid_PLT(mu_iter);
    SS_params.IR_coeff(2) = 0;
    
    try % use existing solution
        SS_params.SS_N = store_mu_PLT{mu_iter}.SS.params.SS_N;
        SS_params.SS_P = store_mu_PLT{mu_iter}.SS.params.SS_P;
    catch % use closest existing solution
        temp = sortrows([1:length(mu_grid_PLT);-abs(mu_grid_PLT-SS_params.mu)]',2);
        for i = 1:size(temp,1)
           try
                SS_params.SS_N = store_mu_PLT{mu_iter}.SS.params.SS_N;
                SS_params.SS_P = store_mu_PLT{mu_iter}.SS.params.SS_P;
           end
        end
    end
  
    [SS_results,SS_policy] = solve_steadystate(SS_params,baseline.SS.policy);  % solve steady state
    

    store_mu_PLT{mu_iter}.SS = SS_results; % store results
    store_mu_PLT{mu_iter}.SS.policy = SS_policy; % store policy functions
end
clear sim_details
save(savename)




%% Analyze solutions
disp('#####################################################')
disp('Analyze solutions and produce figures')
disp('#####################################################')

% Analysis of baseline

% simulate economy with longer time series
options_detail = options;
options_detail.sim_no = 12000;
options_detail.sim_drop = 1000;
options_detail.sim_p = zeros(options_detail.sim_no,1);
options_detail.plotpoints = 500; % subsample of points drawn for each curve when plotting cross-sectional observations
rng default

options_detail.sim_p(1) = round(length(baseline.params.z)/2); % initial productivity state
for t=2:options_detail.sim_no
    options_detail.sim_p(t) = 1+sum(rand>cumsum(baseline.params.z_prob(options_detail.sim_p(t-1),:)));
end

% use policy functions from baseline
grid_N=baseline.policy.grid_N;
x_grid=baseline.policy.x_grid;
aggb=baseline.policy.aggb;
aggq=baseline.policy.aggq;
aggc=baseline.policy.aggc; 
save(options_detail.policy_name,'grid_N','x_grid','aggb','aggq','aggc')

[~,sim_results,sim_details] = het_aggrisk(baseline.params,baseline.coeffs,options_detail);


% store data for figure with accuracy of laws of motion for J and P in baseline
clear fig_acc
fig_acc.J=squeeze(sim_results(:,[3,4])); % perceived and market clearing price of equity
fig_acc.P=squeeze(sim_results(:,[5,6])); % perceived and market clearing price of price level

disp('   produce figures of cross-section in baseline')

% get values for figures 2-9
tempfig = figures_crosssection(sim_details.ind_simulation,sim_results,baseline.params,options_detail);
fig2 = tempfig.fig2;
fig3 = tempfig.fig3;
fig4 = tempfig.fig4;
fig5 = tempfig.fig5;
fig6 = tempfig.fig6;
fig7 = tempfig.fig7;
fig8 = tempfig.fig8;
fig9 = tempfig.fig9;

clear sim_details

% Save IRFs
disp('   save impulse-response functions')

temp_wP = [0.7,1]; % values for wage rigidity that are used
clear fig_IRF
for i=1:length(temp_wP)
    wP_iter = find(wP_grid==temp_wP(i),1);
    fig_IRF{i}.het = store_wP{wP_iter}.IRF;
    fig_IRF{i}.rep = store_wP{wP_iter}.rep.IRF;
end

% Compare solutions with different unemployment insurance
disp('   compare solutions with different unemployment insurance')

% Calculate mean employment depending on UI
temp = sortrows([mu_grid_PLT;1:length(mu_grid_PLT)]',1); % save statistics in ascending order of mu
for i=1:length(mu_grid_PLT)
    mu_iter = temp(i,2);
    figUI1_1_PLT.x(i,:) = mu_grid_PLT(mu_iter); % value of mu
    
    sim_results = store_mu_PLT{mu_iter}.sim_results((options.sim_drop+1):options.sim_no,:); % simulated time series of aggregate variables
    figUI1_1_PLT.y(i,1) = mean(sim_results(sim_results(:,1)<1,2)); % mean employment in recession
    figUI1_1_PLT.y(i,2) = mean(sim_results(sim_results(:,1)>1,2)); % mean employment in boom
    figUI1_1_PLT.y(i,3) = mean(sim_results(:,2)); % unconditional mean employment
    figUI1_1_PLT.y(i,4) = store_mu_PLT{mu_iter}.SS.params.SS_N; % employment without aggregate risk
end


% Plot of time paths in sustained recession
disp('   compute time series in sustained recession')
% Baseline
fig_sustainedIRF{1} =  sustainedIRF(baseline.params,baseline.coeffs,options,trendinflation);
% Without PLT
fig_sustainedIRF{2} =  sustainedIRF(store_PLT{1}.params,store_PLT{1}.coeffs,options,trendinflation);
fig_sustainedIRF{3} = store_alt.IRF;
fig_sustainedIRF{4} = 'time series of sustained recession, 1st cell uses baseline, 2nd cell uses solution with constant nominal interest rate';

%% Save data for figures and generate figures
disp('#####################################################')
disp('Save data for figures and generate figures')
disp('#####################################################')

close all
save([savename,'_figdata'],'fig2','fig3','fig4','fig5','fig6','fig7','fig8','fig9','fig_acc','fig_IRF','fig_sustainedIRF','figUI1_1_PLT')
figures_paper([savename,'_figdata']);


%% Calculate numbers for paper
close all
% long series
[~,sim_results,sim_details] = het_aggrisk(baseline.params,baseline.coeffs,options_detail); % run baseline with long time series


tempt = (options_detail.sim_drop+1):options_detail.sim_no; % periods used
disp('_____________________________')
disp('NUMBERS FOR PAPER:')
disp(['average annualized exp. return of equity: ',num2str((1+mean(sim_results(tempt,14)))^4-1)])
disp(['average annualized real return on bond: ',num2str((1+mean(sim_results(tempt,13)))^4-1)])

% calculate average expected return on portfolio
tempequity = sim_results(tempt-1,3).*sim_results(tempt,2); % total value of equity purchased (one period lagged)
tempbonds = baseline.params.B./sim_results(tempt-1,5).*sim_results(tempt-1,10); % total value of bonds purchased (one period lagged)
tempportfolioreturn = (tempequity.*sim_results(tempt-1,14)+tempbonds.*sim_results(tempt-1,13))./(tempequity+tempbonds);  % expected portfolio return as weighted average of the returns of the two assets
disp(['average annualized real return on portfolio: ',num2str((1+mean(tempportfolioreturn))^4-1)])


tempz = repmat(sim_results(tempt,1),1,options_detail.sim_individual_no); % aggregate productivity replicated for all agents
tempemp = sim_details.ind_simulation.emp(tempt,:); % employment status
tempdeposits2income = sim_details.ind_simulation.deposits2income(tempt,:); % bond holdings to annual income
tempassets2income = sim_details.ind_simulation.assets2income(tempt,:); % financial wealth to annual income
tempz=tempz(:);
tempemp=tempemp(:);
tempdeposits2income=tempdeposits2income(:);
tempassets2income=tempassets2income(:);


% Tables 1 and 2
disp('_____________________________')
disp('TABLES 1 and 2 (parameters and calibration targets')

tempduration = sim_details.ind_simulation.unempduration(tempt-1,:); % duration of unemployment spell (lagged one period to identify last period of spell)
templastperiod = sim_details.ind_simulation.emp(tempt-1,:)==2 & sim_details.ind_simulation.emp(tempt,:)==1; % last period of unemployment spells (lagged one period)
disp(['average length of completed unemployment spell: ',num2str(mean(tempduration(templastperiod)))])
disp(['average unemployment rate: ',num2str(mean(1-sim_results(tempt,2)))])
disp(['median wealth to income ratio of employed during recessions: ',num2str(median(tempassets2income(tempz<1 & tempemp==1)))])

tempvars = log([sim_results(tempt,1).*sim_results(tempt,2),sim_results(tempt,2)]); % log output and employment
[~,tempcycl] = hpfilter(tempvars,1600); % HP-filter
disp(['std(log(q))/std(log(y)) (filtered): ',num2str(std(tempcycl(:,2))/std(tempcycl(:,1)))])

% average relative consumption drop of agents
% consumption in % change to average of 8 quarters consumption before spell and first 4 quarters during unemployment
disp(['consumption drop in first year of unemployment: ',num2str(sim_details.c_drop4(3))]); 
disp(['P change from prolonged boom to 10th quarter of recession: ',num2str(exp(fig_sustainedIRF{1}(11,7))-1)]); 
disp(['P change from prolonged boom to 20th quarter of recession: ',num2str(exp(fig_sustainedIRF{1}(21,7))-1)]); 
disp(['real unit labor cost change from prolonged boom to 20th quarter of recession: ',num2str(exp(fig_sustainedIRF{1}(21,4))-1)]); 
disp(['P change from prolonged boom to 20th quarter of recession without PLT: ',num2str(exp(fig_sustainedIRF{2}(21,7))-1)]); 
disp(['    reduction of (log P change) in prolonged recession with PLT: ', num2str((exp(fig_sustainedIRF{1}(21,7))-1)/(exp(fig_sustainedIRF{2}(21,7))-1)-1)])

disp('PARAMETERS')
disp(['omega_0: ',num2str(baseline.params.w_coeff(1))])
disp(['omega_z: ',num2str(baseline.params.w_coeff(2))])
disp(['omega_P: ',num2str(baseline.params.w_coeff(3))])
disp(['chi: ',num2str(baseline.params.chi)])
disp(['delta: ',num2str(baseline.params.delta)])
disp(['psi: ',num2str(baseline.params.phi0)])
disp(['nu_0: ',num2str(baseline.params.IR_coeff(1))])
disp(['nu_P: ',num2str(baseline.params.IR_coeff(2))])
disp(['mu: ',num2str(baseline.params.mu)])
disp(['beta: ',num2str(baseline.params.beta)])
disp(['money supply: ',num2str(baseline.params.B)])
disp(['gamma: ',num2str(baseline.params.gamma)])
disp(['zeta: ',num2str(baseline.params.zeta)])
disp(['phi1: ',num2str(baseline.params.phi1)])
disp(['z: ',num2str(baseline.params.z)])
disp(['Prob(z''=z): ',num2str(baseline.params.z_prob(1,1))])


% Table 3
disp('_____________________________')
disp('TABLE 3 (median financial wealth to labor income ratios')

% median assets to income unconditional - recession - boom
disp(['median bank deposits/annual income of employed: ',num2str(median(tempdeposits2income(tempz>1 & tempemp==1))),' (expansion); ',num2str(median(tempdeposits2income(tempz<1 & tempemp==1))),' (recession)'])
disp(['median financial assets/annual income of employed: ',num2str(median(tempassets2income(tempz>1 & tempemp==1))),' (expansion); ',num2str(median(tempassets2income(tempz<1 & tempemp==1))),' (recession)'])
disp('_____________________________')

disp(['financial assets/annual income in recession percentiles of employed: ',num2str(prctile(tempassets2income(tempz<1 & tempemp==1),5)),' (5th); ',num2str(prctile(tempassets2income(tempz<1 & tempemp==1),10)),' (10th)'])
disp(['beta in rep agent model: ',num2str(baseline.rep.params.beta)])


disp('_____________________________')
disp('Section 5')
tempwealth = sim_details.ind_simulation.fw(tempt,:); % financial wealth
tempwealth = tempwealth(:);

% calculate mean income loss during unemployment spells
tempincomediff  = (1-baseline.params.mu)*sim_results(:,8).*(1-sim_results(:,9)); % income loss each period when unemployed
tempinds = 1:options_detail.sim_individual_no;
count = 0; % counter for observed unemployment spells
tempincomeloss = [];
for t=tempt
   % identify newly employed in that period
   for j= tempinds(sim_details.ind_simulation.emp(t,:)==1 & sim_details.ind_simulation.emp(t-1,:)==2)
       count = count+1;
       tempincomeloss(count) = sum(tempincomediff((t-sim_details.ind_simulation.unempduration(t-1,j)):(t-1))); % cumulative income loss during that completed spell
   end
end
disp(['(median wealth) to average net-income loss: ',num2str(median(tempwealth(tempemp==1))/mean(tempincomeloss))])

tempunempduration = sim_details.ind_simulation.unempduration(tempt,:); % duration of unemployment spell
tempunempduration = tempunempduration(:);

tempcchange = sim_details.ind_simulation.c(tempt,:)./sim_details.ind_simulation.c(tempt-1,:)-1; % relative change of consumption to previous quarter
tempcchange = tempcchange(:);

tempcoh = sim_details.ind_simulation.fw(tempt,:)+sim_details.ind_simulation.income(tempt,:); % cash-on-hand
tempcoh = tempcoh(:);

disp(['range of consumption drop recessions: ',num2str([min(tempcchange(tempz<1 & tempunempduration==1)),max(tempcchange(tempz<1 & tempunempduration==1))])])
disp(['range of consumption drop expansion: ',num2str([min(tempcchange(tempz>1 & tempunempduration==1)),max(tempcchange(tempz>1 & tempunempduration==1))])])
disp(['average COH newly unemployed: ',num2str(mean(tempcoh(tempz>1 & tempunempduration==1))),' (expansion); ',num2str(mean(tempcoh(tempz<1 & tempunempduration==1))),' (recession)'])
disp(['   change from expansion to recession: ',num2str(mean(tempcoh(tempz<1 & tempunempduration==1))/mean(tempcoh(tempz>1 & tempunempduration==1))-1)])


disp('_____________________________')
disp('Section 6')
disp('IRF baseline')
disp(['  Het: max productivity: ',num2str(exp(min(baseline.IRF(:,1)))-1)])
disp(['  Het: max output: ',num2str(min(exp(baseline.IRF(:,3)))-1)])
disp(['  Rep: max output: ',num2str(min(exp(baseline.rep.IRF(:,3)))-1)])
disp(['  Rep: max real wage: ',num2str(min(exp(baseline.rep.IRF(:,4)))-1)])
disp(['  Het: max employment drop: ',num2str(min(baseline.IRF(:,2)))])
disp(['  Rep: max employment drop: ',num2str(min(baseline.rep.IRF(:,2)))])


tempvars = log([sim_results(tempt,1).*sim_results(tempt,2),sim_results(tempt,8)]); % log output and real wage
[~,tempcycl] = hpfilter(tempvars,1600);
disp(['  correlation of log output and log real wage: ',num2str(corr(tempcycl(:,1),tempcycl(:,2))),' (filtered); ',num2str(corr(tempvars(:,1),tempvars(:,2))),' (unfiltered)'])

tempvars = log([sim_results(tempt,1).*sim_results(tempt,2),sim_results(tempt,8)./sim_results(tempt,1)]); % log output and real unit labor costs
[~,tempcycl] = hpfilter(tempvars,1600);
disp(['  correlation of log output and log real unit labor costs: ',num2str(corr(tempcycl(:,1),tempcycl(:,2))),' (filtered); ',num2str(corr(tempvars(:,1),tempvars(:,2))),' (unfiltered)'])

[~,tempperiod] = min(baseline.rep.IRF(2:end,14));
disp(['  Rep: period of max consumption drop: ',num2str(tempperiod)])
disp('IRF with flexible wages')
disp(['  Rep: max employment: ',num2str((min(store_wP{7}.rep.IRF(:,2))))])
disp(['  Het: max employment: ',num2str((min(store_wP{7}.IRF(:,2))))])



disp('_____________________________')
disp('Section 7')
disp(['limiting value of output drop: ',num2str(exp(fig_sustainedIRF{1}(21,2))-1)])
disp(['limiting value of unemployment rate increase: ',num2str(fig_sustainedIRF{1}(21,8))])
disp(['limiting value of price change: ',num2str(exp(fig_sustainedIRF{1}(21,7))-1)])
disp(['real wage change in 10th quarter: ',num2str(exp(fig_sustainedIRF{1}(11,5))-1)]); 
disp(['average nominal interest rate: ',num2str((mean((1+trendinflation)./sim_results(tempt,10)))^4-1)])
disp(['minimum nominal interest rate: ',num2str((min((1+trendinflation)./sim_results(tempt,10)))^4-1)])
disp(['maximum nominal interest rate: ',num2str((max((1+trendinflation)./sim_results(tempt,10)))^4-1)])

disp('with constant nominal interest rate:')
disp(['    limiting value of real wage: ',num2str(exp(fig_sustainedIRF{2}(21,5))-1)])
disp(['    real wage change in 1st quarter: ',num2str(exp(fig_sustainedIRF{2}(2,5))-1)])

disp('Maximum drop of expected real interest rate')
disp(['  baseline: ',num2str(min(fig_sustainedIRF{1}(:,9)))])
disp(['  no PLT:   ',num2str(min(fig_sustainedIRF{2}(:,9)))])

disp(['Benchmark nominal interest rate end of expansion: ',num2str(fig_sustainedIRF{1}(1,11))])
disp(['Benchmark nominal interest rate initial drop: ',num2str(fig_sustainedIRF{1}(2,11)-fig_sustainedIRF{1}(1,11))])
disp(['Benchmark nominal interest rate drop in quarter 4: ',num2str(fig_sustainedIRF{1}(5,11)-fig_sustainedIRF{1}(1,11))])
disp(['Benchmark nominal interest rate drop in quarter 8: ',num2str(fig_sustainedIRF{1}(9,11)-fig_sustainedIRF{1}(1,11))])

disp(['ZLB average price level in first ten periods relative to end of expansion: ',num2str(mean(exp(fig_sustainedIRF{3}(2:11,7))-1))])

disp('_____________________________')
disp('Section 8')
mu_iter1 = 26; % first value for mu considered (0.45)
mu_iter2 = 6; % second value for mu considered (0.55)
disp(['Change of UI from ',num2str(mu_grid_PLT(mu_iter1)),' to ',num2str(mu_grid_PLT(mu_iter2))])
[~,sim_results_mu1,sim_details_mu1] = het_aggrisk(store_mu_PLT{mu_iter1}.params,store_mu_PLT{mu_iter1}.coeffs,options_detail); % long time-series with first value of mu
[~,sim_results_mu2,sim_details_mu2] = het_aggrisk(store_mu_PLT{mu_iter2}.params,store_mu_PLT{mu_iter2}.coeffs,options_detail); % long time-series with second value of mu

disp(['   change of standard deviation of employment: ',num2str(std(sim_results_mu2(tempt,2))/std(sim_results_mu1(tempt,2))-1)])
disp(['   change of standard deviation of individual consumption: ',num2str(std(reshape(sim_details_mu2.ind_simulation.c(tempt,:),length(tempt)*options_detail.sim_individual_no,1))/std(reshape(sim_details_mu1.ind_simulation.c(tempt,:),length(tempt)*options_detail.sim_individual_no,1))-1)])
disp(['   change of average employment: ',num2str(mean(sim_results_mu2(tempt,2))-mean(sim_results_mu1(tempt,2)))])
disp(['   change of average employment without aggregate risk: ',num2str(store_mu_PLT{mu_iter2}.SS.params.SS_N-store_mu_PLT{mu_iter1}.SS.params.SS_N)])
disp(['   change of standard deviation of real price of equity: ',num2str(std(sim_results_mu2(tempt,3))/std(sim_results_mu1(tempt,3))-1)])
clear('sim_details_mu1','sim_details_mu2')

disp('Decomposition of employment change (in ppt):')
tempeffect(1) = 100*(store_mu_PLT{mu_iter2}.SS.params.SS_N-store_mu_PLT{mu_iter1}.SS.params.SS_N); % change of employment level without aggregate risk
disp(['  1) Level effect without aggregate risk:    ',num2str(tempeffect(1),'%+.2f')])
tempdelta = store_mu_PLT{mu_iter1}.params.delta;
temppsi = store_mu_PLT{mu_iter1}.params.psi;
tempphi0 = store_mu_PLT{mu_iter1}.params.phi0;
tempphi1 = store_mu_PLT{mu_iter1}.params.phi1;
JtoN = @(J)1-tempdelta./(tempdelta+(J/temppsi).^(tempphi1/(1-tempphi1))*tempphi0^(1/(1-tempphi1))); % implied employment by constant J 
tempJ(1:2) = [mean(sim_results_mu1(tempt,3)),mean(sim_results_mu2(tempt,3))]; % average real price of equity with aggregate risk
tempJ_SS(1:2) = [store_mu_PLT{mu_iter1}.SS.stats.SS_J,store_mu_PLT{mu_iter2}.SS.stats.SS_J]; % real price of equity without aggregate risk
tempdJrisk = log(tempJ(2)/tempJ(1))-log(tempJ_SS(2)/tempJ_SS(1)); % change of log J due to risk reduction 
tempeffect(2) = 100*diff(JtoN([tempJ(1),exp(tempdJrisk).*tempJ(1)]),1,2);
disp(['  2) Effect of change in risk on J:          ',num2str(tempeffect(2),'%+.4f')])
disp(['     (change of J without aggregate risk: ',num2str(((tempJ_SS(2)/tempJ_SS(1))'-1),'%.4f'),')'])
disp(['     (change of J with aggregate risk: ',num2str(((tempJ(2)/tempJ(1))'-1),'%.4f'),')'])
tempN(1:2) = [mean(sim_results_mu1(tempt,2)),mean(sim_results_mu2(tempt,2))]; % average employment rate with aggregate risk
tempN_SS(1:2) = [store_mu_PLT{mu_iter1}.SS.params.SS_N,store_mu_PLT{mu_iter2}.SS.params.SS_N]; % employment rate without aggregate risk
tempeffect(3) = -100*diff(JtoN(tempJ)-tempN,1,2);
disp(['  3) Effect of non-linearity in matching:    ',num2str(tempeffect(3),'%+.2f')])
disp(['     (change of N when at mu=',num2str(mu_grid_PLT(mu_iter1)),' when adding aggregate risk: ',num2str(100*(tempN(1)-tempN_SS(1))','%.4f'),')'])
disp(['     (change of N when at mu=',num2str(mu_grid_PLT(mu_iter1)),' due to change in J: ',num2str(100*(JtoN(tempJ(1))-JtoN(tempJ_SS(1)))','%.4f'),')'])
disp(['        ---> not explained: ',num2str(100*(tempN(1)-tempN_SS(1)-JtoN(tempJ(1))+JtoN(tempJ_SS(1)))','%.4f')])
disp(['     (change of N when at mu=',num2str(mu_grid_PLT(mu_iter2)),' when adding aggregate risk: ',num2str(100*(tempN(2)-tempN_SS(2))','%.4f'),')'])
disp(['     (change of N when at mu=',num2str(mu_grid_PLT(mu_iter2)),' due to change in J: ',num2str(100*(JtoN(tempJ(2))-JtoN(tempJ_SS(2)))','%.4f'),')'])
disp(['        ---> not explained: ',num2str(100*(tempN(2)-tempN_SS(2)-JtoN(tempJ(2))+JtoN(tempJ_SS(2)))','%.4f')])
disp('                                             _______')
disp(['  Sum of 3 effects:                          ',num2str(sum(tempeffect),'%+.4f')])
disp(['  Actual change of mean employment:          ',num2str(100*diff(tempN,1,2)','%+.4f')])

clear sim_details
save(savename)


%% Save coefficients
disp('#####################################################')
disp('Save coefficients')
disp('#####################################################')

% only save coefficients
for PLT_iter=1:length(PLT_grid) % solutions with different monetary policy
    temp = store_PLT{PLT_iter}.coeffs;
    store_PLT{PLT_iter}=[];
    store_PLT{PLT_iter}.coeffs = temp;
end

for wP_iter=1:length(wP_grid) % solutions with different nominal wage rigidity
    temp = store_wP{wP_iter}.coeffs;
    if wP_iter == 1 || wP_iter == 7 % baseline and no nominal wage rigidity
        temprep = store_wP{wP_iter}.rep; % add rep agent solution
    end    
  
    store_wP{wP_iter}=[];
    store_wP{wP_iter}.coeffs = temp;
    if wP_iter == 1 || wP_iter == 7 % baseline and no nominal wage rigidity
        store_wP{wP_iter}.rep = temprep; % add rep agent solution
    end     
    
end

for mu_iter=1:length(mu_grid_PLT) % solutions with different unemployment insurance
    temp = store_mu_PLT{mu_iter}.coeffs;
    tempSS  = store_mu_PLT{mu_iter}.SS; % also save solution without aggregate risk
    store_mu_PLT{mu_iter}=[];
    store_mu_PLT{mu_iter}.coeffs = temp;   
    store_mu_PLT{mu_iter}.SS=tempSS;
end

save([savename,'_coeffs'],'baseline','store_PLT','store_alt','store_wP','store_mu_PLT','options','savename')

load(savename) % load complete solutions again

disp('#####################################################')
disp('DONE')
disp('#####################################################')