function err = solve_calib(targets,law_weight,params,x,options)
% function can be used to solve for the coefficients of the aggregate laws
% and to calibrate mean employment or change chi
%
% INPUTS:
%  targets:
%   field N with target for average employment
%	field chi with target value for chi
%  law_weight: weights for coefficients of aggregate law of motion
%  params: structure with parameters
%  x: entries 1-6*z_no are coefficients of aggregate laws, such that
%       coeffs=x(1:6*z_no).*law_weight
%     entry 6*z_no+1 is value for parameter phi0 (optional)
%     entry 6*z_no+2 is value for parameter chi (optional)
%  options: structure with options needed to solve heterogeneous agents
%  with aggregate risk
%
% OUTPUT:
%  err: entries 1-6*z_no are deviations between guess for aggregate laws and
%        estimated new aggregate laws
%       entry 6*z_no+1 deviation of mean employment from target (only calculated when phi0 provided)
%       entry 6*z_no+2 1000*deviation of chi from target (only calculated when phi0 and chi provided)


z_no = length(params.z); % number of possible realizations for aggregate productivity
coeffs=x(1:(6*z_no)).*law_weight; % coefficients of aggregate laws

if length(x)> 6*z_no % if phi0 is provided in x, set params.phi0
    params.phi0 = x(6*z_no+1);
end

if length(x)>6*z_no+1 % if chi is provided in x, set params.chi
    params.chi = x(6*z_no+2);
end

try
    if length(x)>6*z_no % if phi0 is provided in x, also calculate mean employment
        [dev,~,sim_details] = het_aggrisk(params,coeffs,options);
        err(6*z_no+1) = sim_details.meanN(z_no+1)-targets.N;
    else
        dev = het_aggrisk(params,coeffs,options);
    end

    err(1:(6*z_no)) = dev;
    if length(x)>6*z_no+1 % if chi is provided in x, calculate deviation from target
        err(6*z_no+2) = 1000*(params.chi-targets.chi);
    end

catch
    err=NaN*ones(length(x),1); % return NaN, if simulation breaks down
end

