function [SS_results,policy,aggdev,coeffs,sim_results,sim_details] = solve_steadystate(params,policy,options,coeffs)
% Solves for equilibrium without aggregate risk and transition path to
% steady state
%
% INPUTS:
%  params: structure with parameters and SS_N, SS_P as guesses for
%  employment and price level
%  policy: (optional) structure containing
%    x_grid: grid for financial wealth
%    SS_c: guess for policy function for consumption 
%    SS_q: guess for policy function for equity
%    SS_b: guess for policy function for (nominal) bond holdings
%    SS_U: guess for value function (optional)
%      (1st column employed, 2nd column unemployed)
%  options: (optional) structure with various "options" specified needed
%   for het_aggrisk(...). If provided, transition to steady state from
%   initial distribution is sovled
%  coeffs: (optional) initital guess for coefficients in law of motion to
%   steady state
%
% OUTPUTS:
%  SS_results: structure with
%   dev: deviation from market clearing for equity and bonds
%   params: structure with parameters and market-clearing SS_N, SS_P
%   stats: additional statistics calculated in het_steadystate(.)
%  policy: structure containing policy functions for steady state
% if function is called with at least 3 arguments also solve for transition
% to steady state and supply:
%  aggdev: deviation between aggregate laws of motion given by coeffs and
%   estimated new laws using simulated data
%  coeffs: coefficients of aggregate laws in row vector
%  sim_results: simulated time series from het_aggrisk(.)
%  sim_details: detailed statistics from het_aggrisk(.)



if nargin <2 % if policy functions are not provided, create initial guess
    x_min = 0; % lower bound
    x_max = 5; % upper bound
    x_no = 200; % number of grid points
    policy.x_grid = x_min-1+exp(linspace(0,log(x_max-x_min+1),x_no));
    policy.SS_c = 0.5*ones(x_no,2); % policy function for consumption
    policy.SS_q = ones(x_no,2); % policy function for equity
    policy.SS_b = 0.5*ones(x_no,2); % policy function for bond holdings
else
    x_no = size(policy.SS_c,1);
end

% get deviation from market clearing and policy functins
[dev,~,policy] = het_steadystate(params,policy); 
if norm(dev)>1e-8 % if market clearing doesn't hold
    solver_options = optimset('Display','iter','TolFun',1e-8,'TolX',1e-8,'UseParallel',false,'MaxFunEvals',100);
    xsolved = fsolve(@(x)dev_steadystate(x,params,policy),[params.SS_N,params.SS_P],solver_options); % use auxiliary function to solve for N and P
    params.SS_N = xsolved(1);
    params.SS_P = xsolved(2);
end

[dev,stats,policy] = het_steadystate(params,policy);
if norm(dev)>1e-8 
   disp(['###### Warning: steady state not fully converged, dev=',num2str(dev)])
end

SS_results.dev = dev;
SS_results.params = params;
SS_results.stats = stats;


if nargin >= 3 % Also solve transition to steady state
    if nargin< 4 % no initial guess for coefficients provided
        coeffs = zeros(1,6);
        coeffs(1) = log(SS_results.stats.SS_J); 
        coeffs(3) = log(params.SS_P);
    end
    
    N_init = sum(sum(options.init_dist.*options.init_q)); % initial employment given by initial distribution of equity
    try
        grid_Nno = options.grid_Nno;
    catch
        grid_Nno = 10;
    end
    grid_Nspan = sort([N_init,params.SS_N]); % span grid between initial employment and steady state employment
    if diff(grid_Nspan)<0.001
        grid_Nspan = grid_Nspan + [-0.0005,0.0005];
    end
    grid_N = linspace(grid_Nspan(1),grid_Nspan(2),grid_Nno);
    
    % initial guesses for policy functions use steady state policy
    % functions
    aggc = repmat(reshape(policy.SS_c,[1,1,x_no,2]),[1,grid_Nno,1,1]);
    aggb = repmat(reshape(policy.SS_b,[1,1,x_no,2]),[1,grid_Nno,1,1]);
    aggq = repmat(reshape(policy.SS_q,[1,1,x_no,2]),[1,grid_Nno,1,1]);
    x_grid = policy.x_grid;
    save(options.policy_name,'x_grid','aggb','aggq','aggc','grid_N')    
    
    params.meanN = params.SS_N; % reference point in aggregate laws of motion (log(N/meanN))
    % use het_aggrisk(.) to check whether laws of motion are accurate (this
    % is a special case, where z only takes 1 value with probability 1)
    [aggdev,sim_results] = het_aggrisk(params,coeffs,options);
    
    if max(isnan(sim_results(:))) % if initial guess for coefficients results are too bad, use simple guess
        coeffs = zeros(1,6);
        coeffs(1) = log(SS_results.stats.SS_J); 
        coeffs(3) = log(params.SS_P);
        [aggdev,sim_results] = het_aggrisk(params,coeffs,options);
    end
    
    % if the laws have not converged and the actual deviation between
    % perceived and market-clearing values is significant, use solver to
    % find coefficients
    if (norm(aggdev) > options.aggcritvalue) && (norm(log(sim_results(:,[3,5])./sim_results(:,[4,6])))>1e-6) 
        solver_options = optimset('Display','iter','TolFun',1e-6,'TolX',1e-6,'UseParallel',false,'MaxFunEvals',500);

        % when using the solver, put more weight on linear and quadratic coefficients
        % as absolute changes have smaller effects on the solution
        weight=ones(6,1);
        weight([2,4]) = (grid_N(end)-grid_N(1))/2;
        weight(5:6) = ((grid_N(end)-grid_N(1))/2)^2;
        law_weight=1./weight'*0.1;
        
        % solve for laws of motion without recalibrating
        xsolved = fsolve(@(x)solve_calib(0,law_weight,params,x,options),coeffs./law_weight,solver_options);
        coeffs = law_weight.*xsolved;
    end
    
    [aggdev,sim_results,sim_details] = het_aggrisk(params,coeffs,options);
else
    aggdev = NaN;
    coeffs = NaN;
    sim_results = NaN;
    sim_details = NaN;
end




function err =  dev_steadystate(x,params,policy)
% auxiliary function to be used with solver, that replaces guess for
% employment and price level with input from solver
params.SS_N = x(1);
params.SS_P = x(2);

try
    err = het_steadystate(params,policy);
catch
    err=NaN*ones(1,2);
end


